from time import time
from pprint import pprint


def profile(func):
    def wrapper(*args, **kwargs):
        func_name = func.__name__
        start = time()
        result = func(*args, **kwargs)
        duration = time() - start
        metrics = {
            'function_name': func_name,
            'total_duration': humanize_time(duration),
        }
        pprint(metrics)
        return result
    return wrapper


def humanize_time(secs):
    mins, secs = divmod(secs, 60)
    hours, mins = divmod(mins, 60)
    return '%02d:%02d:%02d' % (hours, mins, secs)



import copy
from marshmallow import ValidationError
from sqlalchemy.orm.exc import (
    NoResultFound,
)
from parameterized import parameterized

from test.backendtestcase import TestCase
from test.utils import second_equals_first
from src.cs_models.resources.UserSearchHistoryItem import (
    UserSearchHistoryItem,
)


class UserSearchHistoryItemResourceTestCase(TestCase):
    def setUp(self):
        super(UserSearchHistoryItemResourceTestCase, self).setUp()
        self.inst = UserSearchHistoryItem()

        self.user_id1 = '1000'
        self.user_id2 = '2000'

        self.search_item1 = self.inst.create({
            'user_id': self.user_id1,
            'search_term': 'copaxone',
            'search_type': 'drug',
        })

        self.search_item2 = self.inst.create({
            'user_id': self.user_id1,
            'search_term': 'latuda',
            'search_type': 'drug',
        })

        self.search_item3 = self.inst.create({
            'user_id': self.user_id2,
            'search_term': 'latuda',
            'search_type': 'drug',
        })

        self.search_item4 = self.inst.create({
            'user_id': self.user_id2,
            'search_term': 'teva',
            'search_type': 'company',
        })

        self.valid_data = {
            'user_id': self.user_id1,
            'search_term': 'IPR2015-001',
            'search_type': 'proceeding',
        }

    @parameterized.expand([
        ('user_id',),
        ('search_term',),
    ])
    def test_create_validation_error_missing_field(self, field_to_pop):
        base_data = copy.copy(self.valid_data)
        base_data.pop(field_to_pop)
        self.assertRaises(
            ValidationError,
            self.inst.create,
            base_data,
        )

    def test_create(self):
        resp = self.inst.create(self.valid_data)
        second_equals_first(self.valid_data, resp)

    def test_read(self):
        resp = self.inst.read({})
        self.assertEqual(4, len(resp))

    @parameterized.expand([
        ('id', 'search_item1', 'id', 1),
        ('user_id', 'search_item1', 'user_id', 2),
        ('search_term', 'search_item2', 'search_term', 2),
        ('search_type', 'search_item2', 'search_type', 3),
    ])
    def test_read_w_params(
        self,
        field_name,
        attr,
        attr_field,
        expected_length,
    ):
        resp = self.inst.read({})
        self.assertEqual(4, len(resp))

        resp = self.inst.read({
            field_name: getattr(self, attr)[attr_field],
        })
        self.assertEqual(expected_length, len(resp))

    @parameterized.expand([
        ('id', 999, NoResultFound),
    ])
    def test_one_raises_exception(self, field_name, field_value, exception):
        self.assertRaises(
            exception,
            self.inst.one,
            {
                field_name: field_value,
            },
        )



from marshmallow import (
    Schema,
    fields,
    validate,
    pre_load,
)
from ...utils.utils import pre_load_date_fields


class NCTDesignOutcomesResourceSchema(Schema):
    not_blank = validate.Length(min=1, error="Field cannot be blank")

    id = fields.Integer(dump_only=True)
    nct_study_id = fields.Integer(required=True)
    outcome_type = fields.String(allow_none=True)
    measure = fields.String(allow_none=True)
    time_frame = fields.String(allow_none=True)
    description = fields.String(allow_none=True)
    last_update_submitted_qc_date = fields.DateTime(allow_none=True)
    updated_at = fields.DateTime(dump_only=True)

    @pre_load
    def convert_string_to_datetime(self, in_data, **kwargs):
        date_fields = ['last_update_submitted_qc_date']

        in_data = pre_load_date_fields(
            in_data,
            date_fields,
            date_format='%Y%m%d',
        )
        return in_data



from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm.exc import NoResultFound
from marshmallow import ValidationError, EXCLUDE

from ...database import db_session
from ...resources.CrossRef.models import CrossRefModel
from .schemas import (
    CrossRefResourceSchema,
    CrossRefQueryParamsSchema,
)
from ...utils.utils import update_model


schema_resource = CrossRefResourceSchema()
schema_params = CrossRefQueryParamsSchema()


class CrossRef:
    @staticmethod
    def create(params):
        """
        Args:
            params: Dict (CrossRefResourceSchema)

        Returns:
            CrossRefResourceSchema

        Raises:
            ValidationError
            DBException
        """
        data = schema_resource.load(params, unknown=EXCLUDE)
        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        Args:
            params: CrossRefQueryParamsSchema

        Returns:
            List<CrossRefResourceSchema>

        Raises:
            ValidationError
        """
        data = schema_params.load(params, unknown=EXCLUDE)

        cross_ref_query = _build_query(params=data)
        response = schema_resource.dump(cross_ref_query, many=True)
        return response

    @staticmethod
    def one(params):
        """
        Args:
            params: CrossRefQueryParamsSchema

        Returns:
            CrossRefResourceSchema

        Raises:
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            ValidationError
        """
        data = schema_params.load(params, unknown=EXCLUDE)

        cross_ref_query = _build_query(params=data).one()
        response = schema_resource.dump(cross_ref_query)
        return response

    @staticmethod
    def delete(id):
        """
        :param
            id
        :return:
            delete message
        :exception:
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            SQLAlchemyError
        """
        cross_ref_query = db_session.query(
            CrossRefModel).filter_by(id=id).one()
        try:
            db_session.delete(cross_ref_query)
            db_session.commit()
            db_session.close()
            return 'Successfully deleted'
        except SQLAlchemyError:
            db_session.rollback()
            db_session.close()
            raise

    @staticmethod
    def upsert(params):
        """
        Args:
            params: CrossRefResourceSchema

        Returns:
            CrossRefResourceSchema

        Raises:
            ValidationError
        """
        data = schema_resource.load(params, unknown=EXCLUDE)

        try:
            query_params = {
                'doi': params['doi'],
            }
            cross_ref_query = _build_query(query_params).one()
            response = _helper_update(data, cross_ref_query)
        except NoResultFound:
            response = _helper_create(data)
        return response


def _helper_create(data):
    new_cross_ref = CrossRefModel(**data)
    try:
        db_session.add(new_cross_ref)
        db_session.commit()
        cross_ref_query = db_session.query(
            CrossRefModel).get(new_cross_ref.id)
        response = schema_resource.dump(cross_ref_query)
        db_session.close()
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise


def _helper_update(data, cross_ref_query):
    data['id'] = cross_ref_query.id
    data['doi'] = cross_ref_query.doi
    try:
        update_model(data, cross_ref_query)
        db_session.commit()
        response = schema_resource.dump(cross_ref_query)
        return response
    except SQLAlchemyError:
        db_session.rollback()
        raise


def _build_query(params):
    q = db_session.query(
        CrossRefModel)
    if params.get('id'):
        q = q.filter_by(id=params.get('id'))
    if params.get('doi'):
        q = q.filter_by(doi=params.get('doi'))
    return q



from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm.exc import NoResultFound
from marshmallow import ValidationError, EXCLUDE
from ...database import db_session
from ...resources.FederalCaseRef.models import FederalCaseRefModel
from .schemas import (
    FederalCaseRefResourceSchema,
)
from ...utils.utils import update_model


schema_resource = FederalCaseRefResourceSchema()


class FederalCaseRef(object):
    @staticmethod
    def create(params):
        """
        Args:
            params: Dict (FederalCaseRefResourceSchema)
        Returns:
            FederalCaseRefResourceSchema
        Raises:
            ValidationError
            DBException
        """
        data = schema_resource.load(params, unknown=EXCLUDE)
        response = _helper_create(data)
        return response

    @staticmethod
    def upsert(params):
        """
        Args:
            params: Dict (FederalCaseRefResourceSchema)
        Returns:
            FederalCaseRefResourceSchema
        Raises:
            ValidationError
            DBException
        """
        data = schema_resource.load(params, unknown=EXCLUDE)
        response = _helper_upsert(data)
        return response

    @staticmethod
    def delete(id):
        """
        :param
            id
        :return:
            delete message
        :exception:
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            SQLAlchemyError
        """
        federal_case_ref_query = db_session.query(
            FederalCaseRefModel).filter_by(id=id).one()
        try:
            db_session.delete(federal_case_ref_query)
            db_session.commit()
            db_session.close()
            return 'Successfully deleted'
        except SQLAlchemyError:
            db_session.rollback()
            db_session.close()
            raise


def _helper_create(data):
    new_federal_case_ref = FederalCaseRefModel(**data)
    try:
        db_session.add(new_federal_case_ref)
        db_session.commit()
        federal_case_ref_query = db_session.query(
            FederalCaseRefModel).get(new_federal_case_ref.id)
        response = schema_resource.dump(federal_case_ref_query)
        db_session.close()
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise


def _helper_upsert(data):
    try:
        q = db_session.query(FederalCaseRefModel).filter(
            FederalCaseRefModel.volume == data['volume'],
            FederalCaseRefModel.reporter_type == data['reporter_type'],
            FederalCaseRefModel.page == data['page'],
            FederalCaseRefModel.title == data['title']
        )
        federal_case_ref = q.one()
        # update
        update_model(data, federal_case_ref)
        db_session.commit()
        response = schema_resource.dump(federal_case_ref)
        return response
    except NoResultFound:
        # create
        response = _helper_create(data)
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise



from datetime import datetime
from sqlalchemy import (
    Column,
    Integer,
    String,
    Text,
    DateTime,
    ForeignKey,
    UniqueConstraint,
)

from ...database import Base


class DocumentReferenceModel(Base):
    __tablename__ = "document_references"

    id = Column(Integer, primary_key=True)
    ptab2_document_id = Column(
        Integer, ForeignKey("ptab2_documents.id"), nullable=False,
    )
    type = Column(String(50), nullable=False)
    value = Column(String(128), nullable=False)
    federal_case_headline = Column(String(256))
    federal_case_number = Column(String(256), index=True)
    link = Column(String(500))
    federal_case_file_id = Column(Integer, ForeignKey("files.id"))
    pages = Column(Text)

    updated_at = Column(
        DateTime,
        nullable=False,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )

    __table_args__ = (UniqueConstraint("ptab2_document_id", "type", "value"),)



from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    ForeignKey,
    Boolean,
)
from datetime import datetime

from ...database import Base


class CompanyFilingModel(Base):
    __tablename__ = 'company_filings'

    id = Column(Integer, primary_key=True)
    company_sec_id = Column(
        Integer,
        ForeignKey('companies_sec.id'),
        nullable=True,
    )
    company_ous_id = Column(
        Integer,
        ForeignKey('companies_ous.id'),
        nullable=True,
    )
    file_id = Column(
        Integer,
        ForeignKey('files.id'),
        nullable=False,
    )
    date = Column(DateTime, nullable=True)
    title = Column(String(255), nullable=True)
    type = Column(String(255), nullable=True)
    orig_file_url = Column(String(255), nullable=True)
    is_deleted = Column(Boolean, nullable=True)
    unprocessed = Column(Boolean, nullable=True)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



from marshmallow import (
    Schema,
    fields,
    validate,
)


class CompanyOutboxResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    cik_str = fields.String(allow_none=True)
    ticker = fields.String(allow_none=True)
    name = fields.String(required=True)
    exchange = fields.String(allow_none=True)
    company_sec_id = fields.Integer(allow_none=True)
    company_ous_id = fields.Integer(allow_none=True)
    source = fields.String(required=True)
    news_id = fields.Integer(allow_none=True)
    updated_at = fields.DateTime(dump_only=True)



from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
)
from datetime import datetime

from ...database import Base


class CompanyTickerOutboxModel(Base):
    __tablename__ = 'company_ticker_outbox'

    id = Column(Integer, primary_key=True)
    type = Column(String(10), nullable=False)
    cik_str = Column(String(128), nullable=False)
    ticker = Column(String(20), nullable=False, index=True)
    name = Column(String(255), nullable=False, index=True)
    exchange = Column(String(50), nullable=True)
    updated_at = Column(
        DateTime,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )



from sqlalchemy import (
    Column,
    Integer,
    String,
    Text,
    ForeignKey,
)

from ...aact_database import Base


class StudyReferenceModel(Base):
    __tablename__ = 'study_references'
    __table_args__ = (
        {'schema': 'ctgov'},
    )

    id = Column(Integer, primary_key=True)
    nct_id = Column(
        String,
        ForeignKey('ctgov.studies.nct_id'),
        nullable=False,
    )
    pmid = Column(String)
    reference_type = Column(String)
    citation = Column(Text)

    def __repr__(self):
        return "<StudyReference(id='{}', pmid='{}', reference_type='{}', " \
               "citation='{}'>"\
            .format(self.id, self.pmid, self.reference_type, self.citation)



from datetime import datetime
from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    ForeignKey,
)

from ...database import Base


class PatentIFWModel(Base):
    __tablename__ = 'patent_ifws'

    id = Column(Integer, primary_key=True)
    patent_id = Column(
        Integer,
        ForeignKey('patents.id'),
        nullable=True,
    )
    patent_application_id = Column(
        Integer,
        ForeignKey('patent_applications.id'),
        nullable=False,
    )
    appl_id = Column(String(128), nullable=False)
    document_identifier = Column(String(50), nullable=False)
    document_code = Column(String(50), nullable=True)
    document_description = Column(String(256), nullable=True)
    direction_category = Column(String(50), nullable=True)
    official_date = Column(DateTime, nullable=True)
    total_pages = Column(Integer, nullable=True)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



from marshmallow import (
    Schema,
    fields,
    validate,
    pre_load,
    ValidationError,
)


class WatchlistResourceSchema(Schema):
    not_blank = validate.Length(min=1, error="Field cannot be blank")

    id = fields.Integer(dump_only=True)
    user_id = fields.String(required=True, validate=not_blank)
    resource_type = fields.String(required=True)
    norm_cui = fields.String(allow_none=True)
    condition_id = fields.Integer(allow_none=True)
    company_sec_id = fields.Integer(allow_none=True)
    company_ous_id = fields.Integer(allow_none=True)
    ptab2_proceeding_id = fields.Integer(allow_none=True)
    is_active = fields.Boolean(required=True)
    updated_at = fields.DateTime(dump_only=True)

    @pre_load
    def check_resource_info(self, in_data, **kwargs):
        if self._get_number_of_resource_fields(in_data) == 0:
            raise ValidationError('Provide at least one resource info')
        return in_data

    def _get_number_of_resource_fields(self, in_data, **kwargs):
        result = 0
        if 'norm_cui' in in_data:
            if in_data['norm_cui'] is not None:
                result += 1
        if 'condition_id' in in_data:
            if in_data['condition_id'] is not None:
                result += 1
        if 'company_sec_id' in in_data:
            if in_data['company_sec_id'] is not None:
                result += 1
        if 'company_ous_id' in in_data:
            if in_data['company_ous_id'] is not None:
                result += 1
        if 'ptab2_proceeding_id' in in_data:
            if in_data['ptab2_proceeding_id'] is not None:
                result += 1
        return result



from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    ForeignKey,
    Boolean,
)
from datetime import datetime

from ...database import Base


class WatchlistModel(Base):
    __tablename__ = 'watchlists'

    id = Column(Integer, primary_key=True)
    user_id = Column(String(128), nullable=False)
    resource_type = Column(String(128), nullable=False)
    norm_cui = Column(String(50), nullable=True, index=True)
    condition_id = Column(
        Integer,
        ForeignKey('conditions.id'),
        nullable=True,
    )
    company_sec_id = Column(
        Integer,
        ForeignKey('companies_sec.id'),
        nullable=True,
    )
    company_ous_id = Column(
        Integer,
        ForeignKey('companies_ous.id'),
        nullable=True,
    )
    ptab2_proceeding_id = Column(
        Integer,
        ForeignKey('ptab2_proceedings.id'),
        nullable=True,
    )
    is_active = Column(Boolean, nullable=False)
    updated_at = Column(
        DateTime,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )



from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm.exc import NoResultFound
from marshmallow import ValidationError, EXCLUDE
from ...database import db_session
from ...resources.CompanySEC.models import CompanySECModel
from .schemas import (
    CompanySECResourceSchema,
)
from ...utils.utils import update_model


schema_resource = CompanySECResourceSchema()


class CompanySEC(object):
    @staticmethod
    def create(params):
        """
        Args:
            params: Dict (CompanySECResourceSchema)
        Returns:
            CompanySECResourceSchema
        Raises:
            ValidationError
            DBException
        """
        data = schema_resource.load(params, unknown=EXCLUDE)
        response = _helper_create(data)
        return response

    @staticmethod
    def upsert(params):
        """
        Args:
            params: Dict (CompanySECResourceSchema)
        Returns:
            CompanySECResourceSchema
        Raises:
            ValidationError
            DBException
        """
        data = schema_resource.load(params, unknown=EXCLUDE)
        response = _helper_upsert(data)
        return response

    @staticmethod
    def delete(id):
        """
        :param
            id
        :return:
            delete message
        :exception:
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            SQLAlchemyError
        """
        company_sec_query = db_session.query(
            CompanySECModel).filter_by(id=id).one()
        try:
            db_session.delete(company_sec_query)
            db_session.commit()
            db_session.close()
            return 'Successfully deleted'
        except SQLAlchemyError:
            db_session.rollback()
            db_session.close()
            raise


def _helper_create(data):
    new_company_sec = CompanySECModel(**data)
    try:
        db_session.add(new_company_sec)
        db_session.commit()
        company_sec_query = db_session.query(
            CompanySECModel).get(new_company_sec.id)
        response = schema_resource.dump(company_sec_query)
        db_session.close()
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise


def _helper_upsert(data):
    try:
        q = db_session.query(CompanySECModel).filter(
            CompanySECModel.cik_str == data['cik_str'],
            CompanySECModel.ticker == data['ticker'],
        )
        company_sec = q.one()
        # update
        update_model(data, company_sec)
        db_session.commit()
        response = schema_resource.dump(company_sec)
        return response
    except NoResultFound:
        # create
        response = _helper_create(data)
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise



from datetime import datetime
from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    UniqueConstraint,
)

from ...database import Base


class OrangeBookExclusivityModel(Base):
    __tablename__ = 'orange_book_exclusivities'

    id = Column(Integer, primary_key=True)
    appl_no = Column(String(128), nullable=False)
    appl_type = Column(String(128), nullable=False)
    product_no = Column(String(128), nullable=False)
    exclusivity_code = Column(String(128), nullable=False)
    exclusivity_date = Column(DateTime)
    updated_at = Column(
        DateTime,
        nullable=False,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )

    __table_args__ = (UniqueConstraint(
        'appl_type', 'appl_no', 'product_no', 'exclusivity_code'),)



from marshmallow import (
    Schema,
    fields,
    validate,
    pre_load,
)

from ...utils.utils import pre_load_date_fields


class OrangeBookProductResourceSchema(Schema):
    not_blank = validate.Length(min=1, error="Field cannot be blank")

    id = fields.Integer(dump_only=True)
    appl_no = fields.String(required=True)
    appl_type = fields.String(required=True)
    applicant = fields.String(required=True)
    applicant_full_name = fields.String(required=True)
    applicant_subsidiary_id = fields.Integer(allow_none=True)
    approval_date = fields.DateTime(required=True, allow_none=True)
    dosage_form = fields.String(required=True)
    ingredient = fields.String(required=True)
    product_no = fields.String(required=True)
    rld = fields.String(required=True)
    rs = fields.String(required=True)
    route_of_administration = fields.String(required=True)
    strength = fields.String(required=True)
    te_code = fields.String(required=True)
    trade_name = fields.String(required=True)
    type = fields.String(required=True)
    drug_set_id = fields.String(allow_none=True)

    @pre_load
    def load_date_fields(self, in_data, **kwargs):
        date_fields = ['approval_date']
        in_data = pre_load_date_fields(
            in_data,
            date_fields,
            date_format='%b %d, %Y',
        )
        return in_data


class OrangeBookProductQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error="Field cannot be blank")

    id = fields.Integer()
    appl_type = fields.String(validate=not_blank)
    appl_no = fields.String(validate=not_blank)
    product_no = fields.String(validate=not_blank)
    trade_name = fields.String(validate=not_blank)


class OrangeBookProductPatchSchema(Schema):
    not_blank = validate.Length(min=1, error="Field cannot be blank")

    appl_no = fields.String()
    appl_type = fields.String()
    applicant = fields.String()
    applicant_full_name = fields.String()
    applicant_subsidiary_id = fields.Integer(allow_none=True)
    approval_date = fields.DateTime(allow_none=True)
    dosage_form = fields.String()
    ingredient = fields.String()
    product_no = fields.String()
    rld = fields.String()
    rs = fields.String()
    route_of_administration = fields.String()
    strength = fields.String()
    te_code = fields.String()
    trade_name = fields.String()
    type = fields.String()
    drug_set_id = fields.String(allow_none=True)

    @pre_load
    def load_date_fields(self, in_data, **kwargs):
        date_fields = ['approval_date']
        in_data = pre_load_date_fields(
            in_data,
            date_fields,
            date_format='%b %d, %Y',
        )
        return in_data



from sqlalchemy import (
    Column,
    Integer,
    Text,
    DateTime,
    ForeignKey,
)

from ...database import Base


class AbstractTextModel(Base):
    __tablename__ = 'abstract_texts'

    id = Column(Integer, primary_key=True)
    patent_id = Column(
        Integer,
        ForeignKey('patents.id'),
        nullable=False,
        unique=True,
    )
    abstract_text = Column(Text, nullable=False)
    updated_at = Column(DateTime, nullable=False)



from sqlalchemy.exc import SQLAlchemyError
from datetime import datetime

from ...database import db_session
from ...resources.ClaimDependency.models import (
    ClaimDependencyModel,
)
from .schemas import (
    ClaimDependencyResourceSchema,
    ClaimDependencyQueryParamsSchema,
)
from marshmallow import ValidationError, EXCLUDE


schema_resource = ClaimDependencyResourceSchema()
schema_params = ClaimDependencyQueryParamsSchema()


class DBException(SQLAlchemyError):
    pass


class ClaimDependencyNotFoundException(Exception):
    def __init__(self, message):
        self.message = message


class ClaimDependency:
    @staticmethod
    def create(params):
        """
        :param
            Dict
                claim_id: int
                parent_claim_id: int
        :return:
            newly created object
        :exception:
            ValidationError
            DBException
        """
        data = schema_resource.load(params, unknown=EXCLUDE)

        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        :param
            params: dict
                id
                claim_id
                parent_claim_id
        :return:
            queried object
        :exception:
            ValidationError
        """

        data = schema_params.load(params, unknown=EXCLUDE)

        claim_dependency_query = _build_query(params=data)
        response = schema_resource.dump(claim_dependency_query, many=True)
        return response

    @staticmethod
    def delete(id):
        """
        :param
            id
        :return:
            delete message
        :exception:
            ClaimDependencyNotFoundException
            DBException
        """

        claim_dependency_query = db_session.query(
            ClaimDependencyModel).filter_by(
            id=id).first()
        if not claim_dependency_query:
            raise ClaimDependencyNotFoundException(
                'Claim dependency does not exist!')
        try:
            db_session.delete(claim_dependency_query)
            db_session.commit()
            db_session.close()
            return 'Successfully deleted'
        except SQLAlchemyError:
            db_session.rollback()
            db_session.close()
            raise DBException('DB error')


def _helper_create(data):
    new_claim_dependency = ClaimDependencyModel(
        claim_id=data['claim_id'],
        parent_claim_id=data['parent_claim_id'],
        updated_at=datetime.utcnow(),
    )
    try:
        db_session.add(new_claim_dependency)
        db_session.commit()
        claim_dependency_query = db_session.query(
            ClaimDependencyModel).get(
            new_claim_dependency.id)
        response = schema_resource.dump(claim_dependency_query)
        db_session.close()
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise


def _build_query(params):
    q = db_session.query(ClaimDependencyModel)
    if params.get('id'):
        q = q.filter_by(id=params.get('id'))
    if params.get('claim_id'):
        q = q.filter_by(claim_id=params.get('claim_id'))
    if params.get('parent_claim_id'):
        q = q.filter_by(parent_claim_id=params.get('parent_claim_id'))
    return q



from marshmallow import (
    Schema,
    fields,
    validate,
    pre_load,
)

from ...utils.utils import pre_load_date_fields


class PatentApplicationResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    application_number = fields.String(required=True)
    jurisdiction = fields.String(required=True)
    app_grp_art_number = fields.Integer(allow_none=True)
    abstract_text = fields.String(allow_none=True)
    filed_date = fields.DateTime(allow_none=True)
    inventors = fields.String(allow_none=True)
    applicant = fields.String(allow_none=True)
    title = fields.String(allow_none=True)
    app_class = fields.String(allow_none=True)
    app_sub_class = fields.String(allow_none=True)
    updated_at = fields.DateTime(dump_only=True)

    @pre_load
    def convert_string_to_datetime(self, in_data, **kwargs):
        date_fields = [
            'filed_date',
        ]
        in_data = pre_load_date_fields(
            in_data,
            date_fields,
            date_format='%Y/%m/%d',
        )
        return in_data


class PatentApplicationQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer()
    application_number = fields.String(validate=not_blank)
    jurisdiction = fields.String(validate=not_blank)



import uuid
import json

from sqlalchemy.types import (
    TypeDecorator,
    VARBINARY,
    TEXT,
)
from sqlalchemy.ext import mutable


class UnboundTypeDecorator(TypeDecorator):

    def result_processor(self, dialect, coltype):
        return lambda value: self.process_result_value(value, dialect)

    def bind_processor(self, dialect):
        """ Simple override to avoid pre-processing before
        process_bind_param. This is for when the Python type can't
        be easily coerced into the `impl` type."""
        return lambda value: self.process_bind_param(value, dialect)


class UUID(UnboundTypeDecorator):
    """ A memory-efficient MySQL UUID-type. """

    impl = VARBINARY(16)

    def process_bind_param(self, value, dialect):
        """ Emit the param in hex notation. """
        value = uuid.UUID(str(value))
        return value if value is None else value.bytes

    def process_result_value(self, value, dialect):
        if value is None:
            return value
        else:
            assert len(value) == 16, "Expected 16 bytes, got %d" % len(value)
            return uuid.UUID(bytes=value)


class JsonType(TypeDecorator):
    """Enables JSON storage by encoding and decoding on the fly."""
    impl = TEXT

    def process_bind_param(self, value, dialect):
        if value is None:
            return '{}'
        else:
            return json.dumps(value)

    def process_result_value(self, value, dialect):
        if value is None:
            return {}
        else:
            return json.loads(value)


mutable.MutableDict.associate_with(JsonType)



from datetime import datetime
from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    ForeignKey,
)

from ...database import Base


class OrangeBookProductModel(Base):
    __tablename__ = "orange_book_products"

    id = Column(Integer, primary_key=True)
    appl_no = Column(String(256), index=True)
    appl_type = Column(String(256), index=True)
    applicant = Column(String(256))
    applicant_full_name = Column(String(191))
    applicant_subsidiary_id = Column(
        Integer, ForeignKey("subsidiaries.id"), nullable=True,
    )
    approval_date = Column(DateTime)
    dosage_form = Column(String(256))
    ingredient = Column(String(256))
    product_no = Column(String(128))
    rld = Column(String(256))
    rs = Column(String(256))
    route_of_administration = Column(String(256))
    strength = Column(String(500))
    te_code = Column(String(256))
    trade_name = Column(String(256), index=True)
    type = Column(String(256))
    drug_set_id = Column(String(128), nullable=True)
    updated_at = Column(
        DateTime,
        nullable=False,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )




from sqlalchemy import (
    Column,
    Integer,
    String,
)

from ...aact_database import Base


class MeshTermModel(Base):
    __tablename__ = 'mesh_terms'
    __table_args__ = (
        {'schema': 'ctgov'},
    )

    id = Column(Integer, primary_key=True)
    qualifier = Column(String)
    tree_number = Column(String)
    description = Column(String)
    mesh_term = Column(String)
    downcase_mesh_term = Column(String)

    def __repr__(self):
        return "<MeshTerm(id='{}', qualifier='{}', " \
               "tree_number='{}', description='{}', mesh_term='{}'>"\
            .format(
                self.id,
                self.qualifier,
                self.tree_number,
                self.description,
                self.mesh_term,
            )



from marshmallow import (
    Schema,
    fields,
    validate,
)


class ClaimResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    patent_id = fields.Integer()
    patent_application_id = fields.Integer(required=True)
    claim_number = fields.Integer(required=True)
    claim_text = fields.String(required=True, validate=not_blank)
    updated_at = fields.DateTime(dump_only=True)


class ClaimQueryParamsSchema(Schema):
    id = fields.Integer()
    patent_id = fields.Integer()
    patent_application_id = fields.Integer()
    claim_number = fields.Integer()


class ClaimPatchSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')
    claim_text = fields.String(validate=not_blank)



from sqlalchemy import (
    Integer,
    Column,
    String,
    DateTime,
    ForeignKey
)
from datetime import datetime
from ...database import Base


class NCTFacilityModel(Base):
    __tablename__ = "nct_facilities"

    id = Column(Integer, primary_key=True)
    nct_study_id = Column(
        Integer,
        ForeignKey('nct_study.id'),
        nullable=False,
    )
    facility_name = Column(String(256))
    facility_city = Column(String(50))
    facility_country = Column(String(50))
    facility_zip_code = Column(String(20))
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



from sqlalchemy.exc import SQLAlchemyError
from marshmallow import ValidationError, EXCLUDE

from ...database import db_session
from ...resources.UserSearchHistoryItem.models import (
    UserSearchHistoryItemModel,
)
from .schemas import (
    UserSearchHistoryItemResourceSchema,
    UserSearchHistoryItemQueryParamsSchema,
)


schema_resource = UserSearchHistoryItemResourceSchema()
schema_params = UserSearchHistoryItemQueryParamsSchema()


class UserSearchHistoryItem(object):
    @staticmethod
    def create(params):
        """
        Args:
            params: dict(UserSearchHistoryItemResourceSchema)

        Returns: UserSearchHistoryItemResourceSchema

        Raises:
            ValidationError
            SQLAlchemyError
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)
        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        Args:
            params: dict(UserSearchHistoryItemQueryParamsSchema)

        Returns: List<UserSearchHistoryItemResourceSchema>

        Raises:
            ValidationError
        """
        data = schema_params.load(params, unknown=EXCLUDE)

        user_search_history_item_query = _build_query(params=data)
        response = schema_resource.dump(
            user_search_history_item_query, many=True)
        return response

    @staticmethod
    def one(params):
        """
        Args:
            params: dict(UserSearchHistoryItemQueryParamsSchema)

        Returns: UserSearchHistoryItemResourceSchema

        Raises:
            ValidationError
        """
        data = schema_params.load(params, unknown=EXCLUDE)

        user_search_history_item_query = _build_query(params=data).one()
        response = schema_resource.dump(user_search_history_item_query)
        return response


def _helper_create(data):
    new_search_item = UserSearchHistoryItemModel(**data)
    try:
        db_session.add(new_search_item)
        db_session.commit()
        user_search_history_item_query = db_session.query(
            UserSearchHistoryItemModel,
        ).get(new_search_item.id)
        response = schema_resource.dump(user_search_history_item_query)
        db_session.close()
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise


def _build_query(params):
    q = db_session.query(
        UserSearchHistoryItemModel)
    if params.get('id'):
        q = q.filter_by(id=params.get('id'))
    if params.get('user_id'):
        q = q.filter_by(user_id=params.get('user_id'))
    if params.get('search_term'):
        q = q.filter_by(search_term=params.get('search_term'))
    if params.get('search_type'):
        q = q.filter_by(search_type=params.get('search_type'))
    return q



from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
)
from datetime import datetime

from ...database import Base


class UserSearchHistoryItemModel(Base):
    __tablename__ = 'user_search_history_items'

    id = Column(Integer, primary_key=True)

    user_id = Column(String(128), nullable=False)
    search_term = Column(String(255), nullable=False)
    search_type = Column(String(255))

    updated_at = Column(
        DateTime,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )



from sqlalchemy import (
    Column,
    Integer,
    String,
    Text,
    ForeignKey,
)

from ...aact_database import Base


class DesignOutcomeModel(Base):
    __tablename__ = 'design_outcomes'
    __table_args__ = (
        {'schema': 'ctgov'},
    )

    id = Column(Integer, primary_key=True)
    nct_id = Column(
        String,
        ForeignKey('ctgov.studies.nct_id'),
        nullable=False,
    )
    outcome_type = Column(String)
    measure = Column(Text)
    time_frame = Column(Text)
    population = Column(String)
    description = Column(Text)

    def __repr__(self):
        return "<DesignOutcome(id='{}', nct_id='{}', outcome_type='{}', measure='{}', time_frame='{}', population='{}', description='{}'>"\
            .format(self.id, self.nct_id, self.outcome_type, self.measure, self.time_frame, self.population, self.description)



from marshmallow import ValidationError
from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm.exc import (
    NoResultFound,
    MultipleResultsFound,
)
from parameterized import parameterized

from test.backendtestcase import TestCase
from test.mixins.BasicDataMixin import BasicDataMixin
from test.utils import second_equals_first
from src.cs_models.resources.PacerCase import PacerCase
from src.cs_models.resources.RelatedMatter import RelatedMatter


class RelatedMatterResourceTestCase(BasicDataMixin, TestCase):
    def setUp(self):
        super(RelatedMatterResourceTestCase, self).setUp()
        self.inst_pacer_case = PacerCase()
        self.inst = RelatedMatter()

        self.pacer_case1 = self.inst_pacer_case.create({
            'case_no': '3:17-cv-05314-SDW-LDW',
            'court_id': 'njd',
            'pacer_case_external_id': '2000',
            'cause': '15:1126 Patent Infringement',
        })
        self.pacer_case2 = self.inst_pacer_case.create({
            'case_no': '4:17-cv-05314-SDW-LDW',
            'court_id': 'njd',
            'pacer_case_external_id': '3000',
            'cause': '15:1126 Patent Infringement',
        })
        self.pacer_case3 = self.inst_pacer_case.create({
            'case_no': '5:17-cv-05314-SDW-LDW',
            'court_id': 'njd',
            'pacer_case_external_id': '4000',
            'cause': '15:1126 Patent Infringement',
        })

        self.ptab2_proceeding3 = self.inst_ptab2_proceeding.create({
            'accorded_filing_date': '05-06-2013',
            'decision_date': '10-07-2014',
            'institution_decision_date': '10-08-2013',
            'petitioner_application_number_text': '-',
            'petitioner_counsel_name': 'Steven Baughman',
            'petitioner_grant_date': '-',
            'petitioner_group_art_unit_number': '-',
            'petitioner_inventor_name': '-',
            'petitioner_party_name': 'Apple Inc.',
            'petitioner_patent_number': '-',
            'petitioner_patent_owner_name': '-',
            'petitioner_technology_center_number': '-',
            'proceeding_filing_date': '05-06-2013',
            'proceeding_last_modified_date': '-',
            'proceeding_number': 'IPR-30033',
            'proceeding_status_category': 'FWD Entered',
            'proceeding_type_category': 'AIA Trial',
            'respondent_application_number_text': '08471964',
            'respondent_counsel_name': 'David Marsh',
            'respondent_grant_date': '10-12-1999',
            'respondent_group_art_unit_number': '-',
            'respondent_inventor_name': 'ARTHUR HAIR',
            'respondent_party_name': 'SightSound Technologies LLC',
            'respondent_patent_number': '5966440',
            'respondent_patent_owner_name': 'SightSound Technologies LLC',
            'respondent_technology_center_number': '2100',
            'subproceeding_type_category': 'CBM',
            'proceeding_status': None,
        })

        self.related_matter1 = self.inst.create({
            'ptab2_document_id': self.ptab2_document1['id'],
            'related_pacer_case_id': self.pacer_case1['id'],
        })

        self.related_matter2 = self.inst.create({
            'ptab2_document_id': self.ptab2_document1['id'],
            'related_pacer_case_id': self.pacer_case2['id'],
        })

        self.related_matter3 = self.inst.create({
            'ptab2_document_id': self.ptab2_document1['id'],
            'related_ptab2_proceeding_id': self.ptab2_proceeding2['id'],
        })

    def _get_dict_with_fields(self, base_data, fields_to_keep):
        data = {}
        for key in base_data:
            if key in fields_to_keep:
                data[key] = base_data[key]
        return data

    @parameterized.expand([
        (['ptab2_document_id'],),
        (['related_pacer_case_id', 'related_ptab2_proceeding_id'],),
    ])
    def test_create_validation_error_missing_field(self, fields_to_keep):
        base_data = {
            'ptab2_document_id': self.ptab2_document1['id'],
            'related_pacer_case_id': self.pacer_case3['id'],
            'related_ptab2_proceeding_id': self.ptab2_proceeding3['id'],
        }
        data = self._get_dict_with_fields(base_data, fields_to_keep)

        self.assertRaises(
            ValidationError,
            self.inst.create,
            data,
        )

    def test_create_validation_error_both_fields(self):
        data = {
            'ptab2_document_id': self.ptab2_document1['id'],
            'related_pacer_case_id': self.pacer_case3['id'],
            'related_ptab2_proceeding_id': self.ptab2_proceeding3['id'],
        }
        self.assertRaises(
            ValidationError,
            self.inst.create,
            data,
        )

    @parameterized.expand([
        (['ptab2_document_id', 'related_pacer_case_id'],),
        (['ptab2_document_id', 'related_ptab2_proceeding_id'],),
    ])
    def test_create_violates_unique_constraint(self, fields_to_keep):
        base_data = {
            'ptab2_document_id': self.ptab2_document1['id'],
            'related_pacer_case_id': self.pacer_case2['id'],
            'related_ptab2_proceeding_id': self.ptab2_proceeding2['id'],
        }
        data = self._get_dict_with_fields(base_data, fields_to_keep)

        self.assertRaises(
            IntegrityError,
            self.inst.create,
            data,
        )

    @parameterized.expand([
        ('ptab2_document_id', ['related_pacer_case_id']),
        ('related_pacer_case_id', ['ptab2_document_id', 'related_pacer_case_id']),
        ('related_ptab2_proceeding_id', ['ptab2_document_id', 'related_ptab2_proceeding_id']),
    ])
    def test_create_violates_fk_constraint(
        self,
        field_to_test,
        fields_to_keep,
    ):
        base_data = {
            'ptab2_document_id': self.ptab2_document1['id'],
            'related_pacer_case_id': self.pacer_case2['id'],
            'related_ptab2_proceeding_id': self.ptab2_proceeding2['id'],
        }
        data = self._get_dict_with_fields(base_data, fields_to_keep)
        data[field_to_test] = 999

        self.assertRaises(
            IntegrityError,
            self.inst.create,
            data,
        )

    @parameterized.expand([
        (['ptab2_document_id', 'related_pacer_case_id'],),
        (['ptab2_document_id', 'related_ptab2_proceeding_id'],),
    ])
    def test_create(self, fields_to_keep):
        base_data = {
            'ptab2_document_id': self.ptab2_document1['id'],
            'related_pacer_case_id': self.pacer_case3['id'],
            'related_ptab2_proceeding_id': self.ptab2_proceeding3['id'],
        }
        data = self._get_dict_with_fields(base_data, fields_to_keep)
        resp = self.inst.create(data)
        second_equals_first(data, resp)

    def test_read(self):
        resp = self.inst.read({})
        self.assertEqual(3, len(resp))

    @parameterized.expand([
        ('id', 'related_matter1', 'id', 1),
        ('ptab2_document_id', 'ptab2_document1', 'id', 3),
        ('related_pacer_case_id', 'pacer_case1', 'id', 1),
        ('related_ptab2_proceeding_id', 'ptab2_proceeding2', 'id', 1),
    ])
    def test_read_w_params(
        self,
        field_name,
        attr,
        attr_field,
        expected_length,
    ):
        resp = self.inst.read({})
        self.assertEqual(len(resp), 3)

        resp = self.inst.read({
            field_name: getattr(self, attr)[attr_field],
        })
        self.assertEqual(expected_length, len(resp))

    def test_one_raises_no_result_found_exception(self):
        self.assertRaises(
            NoResultFound,
            self.inst.one,
            {'id': 9999},
        )

    def test_one_raises_multiple_results_found_exception(self):
        self.assertRaises(
            MultipleResultsFound,
            self.inst.one,
            {'ptab2_document_id': self.ptab2_document1['id']},
        )

    @parameterized.expand([
        ('id', 'related_matter1'),
        ('related_pacer_case_id', 'related_matter1'),
        ('related_ptab2_proceeding_id', 'related_matter3'),
    ])
    def test_one(self, field_name, attr):
        related_matter = getattr(self, attr)
        resp = self.inst.one({
            field_name: related_matter[field_name],
        })
        second_equals_first(
            related_matter,
            resp,
        )

    def test_delete_not_found(self):
        invalid_id = 99999
        self.assertRaises(
            NoResultFound,
            self.inst.delete,
            invalid_id,
        )

    def test_delete(self):
        response = self.inst.one({'id': self.related_matter1['id']})
        self.inst.delete(id=response['id'])
        self.assertRaises(
            NoResultFound,
            self.inst.one,
            {'id': self.related_matter1['id']},
        )



import copy

from marshmallow import ValidationError
from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm import Query
from parameterized import parameterized

from test.backendtestcase import TestCase
from test.utils import second_equals_first
from src.cs_models.resources.CompanySEC import (
    CompanySEC,
)
from src.cs_models.resources.CompanySEC.models import (
    CompanySECModel,
)
from src.cs_models.database import db_session


class CompanySECResourceTestCase(TestCase):
    def setUp(self):
        super(CompanySECResourceTestCase, self).setUp()
        self.inst = CompanySEC()

        self.company_sec1 = self.inst.create({
            'cik_str': '1652044',
            'ticker': 'GOOG',
            'title': 'Alphabet Inc.',
        })

        self.valid_data = {
            'cik_str': '1577552',
            'ticker': 'BABA',
            'title': 'Alibaba Group Holding Ltd',
        }

    @parameterized.expand([
        ('cik_str',),
        ('title',),
    ])
    def test_create_validation_error_missing_field(self, field_to_pop):
        base_data = copy.copy(self.valid_data)
        base_data.pop(field_to_pop)
        self.assertRaises(
            ValidationError,
            self.inst.create,
            base_data,
        )

    def test_create_violates_unique_constraint(self):
        self.assertRaises(
            IntegrityError,
            self.inst.create,
            {
                'cik_str': self.company_sec1['cik_str'],
                'ticker': self.company_sec1['ticker'],
                'title': self.company_sec1['title'],
            },
        )

    def test_create(self):
        resp = self.inst.create(self.valid_data)
        second_equals_first(
            self.valid_data,
            resp,
        )

    def test_delete(self):
        self.inst.delete(id=self.company_sec1['id'])

        q = self._build_query(company_sec_id=self.company_sec1['id'])

        assert q.first() is None

    def test_upsert_creates_new_row(self):
        q = db_session.query(CompanySECModel)
        assert len(q.all()) == 1
        self.inst.upsert(self.valid_data)
        q = db_session.query(CompanySECModel)
        assert len(q.all()) == 2

    def test_upsert_updates_existing_row(self):
        data = {
            'cik_str': self.company_sec1['cik_str'],
            'ticker': self.company_sec1['ticker'],
            'title': 'Google Inc.',
        }
        q = db_session.query(CompanySECModel)
        assert len(q.all()) == 1
        response = self.inst.upsert(data)
        q = db_session.query(CompanySECModel)
        assert len(q.all()) == 1
        second_equals_first(data, response)

    @parameterized.expand([
        ('cik_str',),
        ('title',),
    ])
    def test_upsert_raises_validation_error(self, field_to_pop):
        base_data = copy.copy(self.valid_data)
        base_data.pop(field_to_pop)
        self.assertRaises(
            ValidationError,
            self.inst.upsert,
            base_data,
        )


    @staticmethod
    def _build_query(company_sec_id: int) -> Query:
        q = db_session.query(
            CompanySECModel,
        ).filter_by(id=company_sec_id)
        return q



from marshmallow import (
    Schema,
    fields,
    validate,
    pre_load,
    ValidationError,
)
from ...utils.utils import pre_load_date_fields


class MergerResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    target_sec_id = fields.Integer(allow_none=True)
    target_ous_id = fields.Integer(allow_none=True)
    acquirer_sec_id = fields.Integer(allow_none=True)
    acquirer_ous_id = fields.Integer(allow_none=True)
    deal_value = fields.Decimal()
    type = fields.String()
    announcement_date = fields.DateTime(required=True)
    offer_price = fields.Float()
    market_price = fields.Float()
    dma_file_id = fields.Integer()
    updated_at = fields.DateTime()

    @pre_load
    def convert_string_to_datetime(self, in_data, **kwargs):
        date_fields = ['announcement_date']

        in_data = pre_load_date_fields(
            in_data,
            date_fields,
            date_format='%m/%d/%Y',
        )
        return in_data

    @pre_load
    def check_target_acquirer_ids(self, in_data, **kwargs):
        if self._get_number_of_target_company_fields(in_data) != 1:
            raise ValidationError('Provide either company_sec_id or '
                                  'company_ous_id for target, not both')
        if self._get_number_of_acquirer_company_fields(in_data) != 1:
            raise ValidationError('Provide either company_sec_id or '
                                  'company_ous_id for acquirer, not both')
        return in_data

    def _get_number_of_target_company_fields(self, in_data, **kwargs):
        result = 0
        if 'target_sec_id' in in_data:
            if in_data['target_sec_id'] is not None:
                result += 1
        if 'target_ous_id' in in_data:
            if in_data['target_ous_id'] is not None:
                result += 1
        return result

    def _get_number_of_acquirer_company_fields(self, in_data, **kwargs):
        result = 0
        if 'acquirer_sec_id' in in_data:
            if in_data['acquirer_sec_id'] is not None:
                result += 1
        if 'acquirer_ous_id' in in_data:
            if in_data['acquirer_ous_id'] is not None:
                result += 1
        return result


class MergerQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer()
    target_sec_id = fields.Integer()
    target_ous_id = fields.Integer()
    acquirer_sec_id = fields.Integer()
    acquirer_ous_id = fields.Integer()
    deal_value = fields.Decimal()
    type = fields.String()
    announcement_date = fields.DateTime()

    @pre_load
    def convert_string_to_datetime(self, in_data, **kwargs):
        date_fields = ['announcement_date']

        in_data = pre_load_date_fields(
            in_data,
            date_fields,
            date_format='%m/%d/%Y',
        )
        return in_data


class MergerPatchSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    target_sec_id = fields.Integer()
    target_ous_id = fields.Integer()
    acquirer_sec_id = fields.Integer()
    acquirer_ous_id = fields.Integer()
    deal_value = fields.Decimal()
    type = fields.String()
    announcement_date = fields.DateTime(required=True)
    offer_price = fields.Float()
    market_price = fields.Float()
    dma_file_id = fields.Integer()

    @pre_load
    def convert_string_to_datetime(self, in_data, **kwargs):
        date_fields = ['announcement_date']

        in_data = pre_load_date_fields(
            in_data,
            date_fields,
            date_format='%m/%d/%Y',
        )
        return in_data

    @pre_load
    def check_target_acquirer_ids(self, in_data, **kwargs):
        if self._get_number_of_target_company_fields(in_data) != 1:
            raise ValidationError('Provide either company_sec_id or '
                                  'company_ous_id for target, not both')
        if self._get_number_of_acquirer_company_fields(in_data) != 1:
            raise ValidationError('Provide either company_sec_id or '
                                  'company_ous_id for acquirer, not both')
        return in_data

    def _get_number_of_target_company_fields(self, in_data, **kwargs):
        result = 0
        if 'target_sec_id' in in_data:
            if in_data['target_sec_id'] is not None:
                result += 1
        if 'target_ous_id' in in_data:
            if in_data['target_ous_id'] is not None:
                result += 1
        return result

    def _get_number_of_acquirer_company_fields(self, in_data, **kwargs):
        result = 0
        if 'acquirer_sec_id' in in_data:
            if in_data['acquirer_sec_id'] is not None:
                result += 1
        if 'acquirer_ous_id' in in_data:
            if in_data['acquirer_ous_id'] is not None:
                result += 1
        return result



from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    ForeignKey,
    Boolean,
    Text,
)
from datetime import datetime

from ...database import Base


class CompanyReviewModel(Base):
    __tablename__ = 'company_review'

    id = Column(Integer, primary_key=True)
    company_sec_id = Column(
        Integer,
        ForeignKey('companies_sec.id'),
        nullable=True,
    )
    company_ous_id = Column(
        Integer,
        ForeignKey('companies_ous.id'),
        nullable=True,
    )
    note_tag = Column(Text, nullable=True)
    news_id = Column(
        Integer,
        ForeignKey('newswires.id'),
        nullable=False,
    )
    reviewed = Column(Boolean, nullable=False)
    updated_at = Column(
        DateTime,
        nullable=False,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )



from marshmallow import (
    Schema,
    fields,
    pre_load,
    validate,
)
from ...utils.utils import pre_load_date_fields


class LicensingCollabOutboxResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    date = fields.DateTime(required=True)
    news_id = fields.Integer(required=True)
    upfronts = fields.String(allow_none=True)
    milestones = fields.String(allow_none=True)
    sentences = fields.String(allow_none=True)
    updated_at = fields.DateTime()

    @pre_load
    def convert_string_to_datetime(self, in_data, **kwargs):
        date_fields = ['date']

        in_data = pre_load_date_fields(
            in_data,
            date_fields,
            date_format='%Y%m%dT%H%M%S',
        )
        return in_data



from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm.exc import NoResultFound
from marshmallow import ValidationError, EXCLUDE

from ...database import db_session
from ...resources.PTAB2Decision.models import PTAB2DecisionModel
from .schemas import (
    PTAB2DecisionResourceSchema,
    PTAB2DecisionQueryParamsSchema,
    PTAB2DecisionPatchSchema,
)
from ...utils.utils import update_model


schema_resource = PTAB2DecisionResourceSchema()
schema_params = PTAB2DecisionQueryParamsSchema()
schema_patch = PTAB2DecisionPatchSchema()


class PTAB2Decision(object):
    @staticmethod
    def create(params):
        """
        Args:
            params: Dict (PTAB2DecisionResourceSchema)

        Returns:
            PTAB2DecisionResourceSchema

        Raises:
            ValidationError
            DBException
        """
        data = schema_resource.load(params, unknown=EXCLUDE)
        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        Args:
            params: PTAB2DecisionQueryParamsSchema

        Returns:
            List<PTAB2DecisionResourceSchema>

        Raises:
            ValidationError
        """
        data = schema_params.load(params, unknown=EXCLUDE)

        ptab2_decision_query = _build_query(params=data)
        response = schema_resource.dump(ptab2_decision_query, many=True)
        return response

    @staticmethod
    def one(params):
        """
        Args:
            params: PTAB2DecisionQueryParamsSchema

        Returns:
            PTAB2DecisionResourceSchema

        Raises:
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            ValidationError
        """
        data = schema_params.load(params, unknown=EXCLUDE)

        ptab2_decision_query = _build_query(params=data).one()
        response = schema_resource.dump(ptab2_decision_query)
        return response

    @staticmethod
    def delete(id):
        """
        :param
            id
        :return:
            delete message
        :exception:
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            SQLAlchemyError
        """
        ptab2_decision_query = db_session.query(
            PTAB2DecisionModel).filter_by(id=id).one()
        try:
            db_session.delete(ptab2_decision_query)
            db_session.commit()
            db_session.close()
            return 'Successfully deleted'
        except SQLAlchemyError:
            db_session.rollback()
            db_session.close()
            raise

    @staticmethod
    def upsert(params):
        """
        Args:
            params: PTAB2DecisionResourceSchema

        Returns:
            PTAB2DecisionResourceSchema

        Raises:
            ValidationError
        """
        data = schema_resource.load(params, unknown=EXCLUDE)

        try:
            query_params = {
                'identifier': params['identifier'],
            }
            ptab2_decision_query = _build_query(query_params).one()
            response = _helper_update(data, ptab2_decision_query)
        except NoResultFound:
            response = _helper_create(data)
        return response

    @staticmethod
    def update(id, params):
        """

        Args:
            id: int
            params: PTAB2DecisionPatchSchema

        Returns:
            PTAB2DecisionResourceSchema

        Raises:
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            ValidationError
        """
        ptab2_decision_query = db_session.query(
            PTAB2DecisionModel).filter_by(id=id).one()
        data = schema_patch.load(params, unknown=EXCLUDE)

        response = _helper_update(data, ptab2_decision_query)
        return response


def _helper_create(data):
    new_ptab2_decision = PTAB2DecisionModel(**data)
    try:
        db_session.add(new_ptab2_decision)
        db_session.commit()
        ptab2_decision_query = db_session.query(
            PTAB2DecisionModel).get(new_ptab2_decision.id)
        response = schema_resource.dump(ptab2_decision_query)
        db_session.close()
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise


def _helper_update(data, ptab2_decision_query):
    data['id'] = ptab2_decision_query.id
    data['identifier'] = ptab2_decision_query.identifier
    try:
        update_model(data, ptab2_decision_query)
        db_session.commit()
        response = schema_resource.dump(ptab2_decision_query)
        return response
    except SQLAlchemyError:
        db_session.rollback()
        raise


def _build_query(params):
    q = db_session.query(
        PTAB2DecisionModel)
    if params.get('id'):
        q = q.filter_by(id=params.get('id'))
    if params.get('identifier'):
        q = q.filter_by(identifier=params.get('identifier'))
    return q



from sqlalchemy import (
    Column,
    Integer,
    String,
    Date,
    Text,
    DateTime,
    Boolean,
    PrimaryKeyConstraint,
)

from ...aact_database import Base


class StudyModel(Base):

    __tablename__ = 'studies'
    __table_args__ = (
        PrimaryKeyConstraint('nct_id'),
        {'schema': 'ctgov'},
    )

    nct_id = Column(String, primary_key=True)
    nlm_download_date_description = Column(String)
    study_first_submitted_date = Column(Date)
    last_update_submitted_date = Column(Date)
    results_first_submitted_date = Column(Date)
    disposition_first_submitted_date = Column(Date)
    start_month_year = Column(String)
    start_date = Column(Date)
    start_date_type = Column(String)
    verification_month_year = Column(String)
    verification_date = Column(Date)
    completion_month_year = Column(String)
    completion_date = Column(Date)
    completion_date_type = Column(String)
    primary_completion_month_year = Column(String)
    primary_completion_date = Column(Date)
    primary_completion_date_type = Column(String)
    study_first_submitted_qc_date = Column(Date)
    study_first_posted_date = Column(Date)
    study_first_posted_date_type = Column(String)
    results_first_submitted_qc_date = Column(Date)
    results_first_posted_date = Column(Date)
    results_first_posted_date_type = Column(String)
    disposition_first_submitted_qc_date = Column(Date)
    disposition_first_posted_date = Column(Date)
    disposition_first_posted_date_type = Column(String)
    last_update_submitted_qc_date = Column(Date)
    last_update_posted_date = Column(Date)
    last_update_posted_date_type = Column(String)
    target_duration = Column(String)
    study_type = Column(String)
    acronym = Column(String)
    baseline_population = Column(Text)
    brief_title = Column(Text)
    official_title = Column(Text)
    overall_status = Column(String)
    last_known_status = Column(String)
    phase = Column(String)
    enrollment = Column(Integer)
    enrollment_type = Column(String)
    source = Column(String)
    limitations_and_caveats = Column(String)
    number_of_arms = Column(Integer)
    number_of_groups = Column(Integer)
    why_stopped = Column(String)
    has_expanded_access = Column(Boolean)
    expanded_access_type_individual = Column(Boolean)
    expanded_access_type_intermediate = Column(Boolean)
    expanded_access_type_treatment = Column(Boolean)
    has_dmc = Column(Boolean)
    is_fda_regulated_drug = Column(Boolean)
    is_fda_regulated_device = Column(Boolean)
    is_unapproved_device = Column(Boolean)
    is_ppsd = Column(Boolean)
    is_us_export = Column(Boolean)
    biospec_retention = Column(String)
    biospec_description = Column(Text)
    plan_to_share_ipd = Column(String)
    plan_to_share_ipd_description = Column(String)
    ipd_time_frame = Column(String)
    ipd_access_criteria = Column(String)
    ipd_url = Column(String)
    created_at = Column(DateTime)
    updated_at = Column(DateTime)

    def __repr__(self):
        return "<Study(nct_id='{}', official_title='{}', phase={}, " \
               "start_date={})>".format(self.nct_id,
                                        self.official_title,
                                        self.phase,
                                        self.start_date,
                                        )



from marshmallow import (
    Schema,
    fields,
    validate,
    pre_load,
)
from ...utils.utils import pre_load_date_fields


class NewswireResourceSchema(Schema):
    not_blank = validate.Length(min=1, error="Field cannot be blank")

    id = fields.Integer(dump_only=True)
    date = fields.DateTime(required=True)
    tzinfo = fields.String(allow_none=True)
    source = fields.String(required=True)
    headline = fields.String(required=True)
    drugs = fields.String(allow_none=True)
    conditions = fields.String(allow_none=True)
    concepts = fields.String(allow_none=True)
    filtered_drugs = fields.String(allow_none=True)
    filtered_conditions = fields.String(allow_none=True)
    news_file_id = fields.Integer(required=True)
    updated_at = fields.DateTime(dump_only=True)

    @pre_load
    def convert_string_to_datetime(self, in_data, **kwargs):
        date_fields = ['date']

        in_data = pre_load_date_fields(
            in_data,
            date_fields,
            date_format='%Y%m%dT%H%M%S',
        )
        return in_data


class NewswireQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error="Field cannot be blank")

    id = fields.Integer()
    date = fields.DateTime()
    tzinfo = fields.String()
    source = fields.String()
    headline = fields.String()
    drugs = fields.String()
    conditions = fields.String()
    concepts = fields.String()
    filtered_drugs = fields.String()
    filtered_conditions = fields.String()

    @pre_load
    def convert_string_to_datetime(self, in_data, **kwargs):
        date_fields = ['date']

        in_data = pre_load_date_fields(
            in_data,
            date_fields,
            date_format='%Y%m%dT%H%M%S',
        )
        return in_data


class NewswirePatchSchema(Schema):
    not_blank = validate.Length(min=1, error="Field cannot be blank")

    date = fields.DateTime(allow_none=False)
    tzinfo = fields.String()
    source = fields.String(allow_none=False)
    headline = fields.String(allow_none=False)
    drugs = fields.String(allow_none=True)
    conditions = fields.String(allow_none=True)
    concepts = fields.String(allow_none=True)
    filtered_drugs = fields.String(allow_none=True)
    filtered_conditions = fields.String(allow_none=True)
    news_file_id = fields.Integer(allow_none=True)

    @pre_load
    def convert_string_to_datetime(self, in_data, **kwargs):
        date_fields = ['date']

        in_data = pre_load_date_fields(
            in_data,
            date_fields,
            date_format='%Y%m%dT%H%M%S',
        )
        return in_data


from marshmallow import (
    Schema,
    fields,
)


class NDCSetIDCrossResourceSchema(Schema):
    product_ndc = fields.String(allow_none=True)
    package_ndc = fields.String(allow_none=True)
    spl_set_id = fields.String(allow_none=True)


class NDCSetIDCrossQueryParamsSchema(Schema):
    product_ndc = fields.String(allow_none=True)
    package_ndc = fields.String(allow_none=True)
    spl_set_id = fields.String(allow_none=True)


class NDCSetIDCrossPatchSchema(Schema):
    product_ndc = fields.String(allow_none=True)
    package_ndc = fields.String(allow_none=True)
    spl_set_id = fields.String(allow_none=True)




from sqlalchemy.exc import SQLAlchemyError
from datetime import datetime
from marshmallow import ValidationError, EXCLUDE

from ...database import db_session
from ...resources.PatentCompound.models import (
    PatentCompoundModel,
)
from .schemas import (
    PatentCompoundResourceSchema,
    PatentCompoundQueryParamsSchema,
)


schema_resource = PatentCompoundResourceSchema()
schema_params = PatentCompoundQueryParamsSchema()


class DBException(SQLAlchemyError):
    pass


class PatentCompoundNotFoundException(Exception):
    def __init__(self, message):
        self.message = message


class PatentCompound:
    @staticmethod
    def create(params):
        """
        :param
            Dict: PatentCompoundResourceSchema
        :return:
            PatentCompoundResourceSchema
        :exception:
            ValidationError
            DBException
        """
        data = schema_resource.load(params, unknown=EXCLUDE)
        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        :param
            params: PatentCompoundQuerySchema
        :return:
            List<PatentCompoundResourceSchema>
        :exception:
            ValidationError
        """

        data = schema_params.load(params, unknown=EXCLUDE)

        patent_query = _build_query(params=data)
        response = schema_resource.dump(patent_query, many=True)
        return response

    @staticmethod
    def one(params):
        """
        :param
            params: PatentCompoundQuerySchema
        :return:
            PatentCompoundResourceSchema
        :exception:
            ValidationError
        """

        data = schema_params.load(params, unknown=EXCLUDE)

        patent_query = _build_query(params=data).one()
        response = schema_resource.dump(patent_query)
        return response


def _helper_create(data):
    new_patent = PatentCompoundModel(
        updated_at=datetime.utcnow(),
        **data,
    )
    try:
        db_session.add(new_patent)
        db_session.commit()
        patent_query = db_session.query(
            PatentCompoundModel).get(new_patent.id)
        response = schema_resource.dump(patent_query)
        db_session.close()
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise


def _build_query(params):
    q = db_session.query(PatentCompoundModel)
    if params.get('id'):
        q = q.filter_by(id=params.get('id'))
    if params.get('patent_id'):
        q = q.filter_by(patent_id=params.get('patent_id'))
    if params.get('cid'):
        q = q.filter_by(cid=params.get('cid'))
    if params.get('compound_name'):
        q = q.filter_by(compound_name=params.get('compound_name'))
    if params.get('iupac_name'):
        q = q.filter_by(iupac_name=params.get('iupac_name'))

    return q



from marshmallow import (
    Schema,
    fields,
    validate,
)


class AbstractTextResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    patent_id = fields.Integer()
    abstract_text = fields.String()
    updated_at = fields.DateTime()


class AbstractTextQueryParamsSchema(Schema):
    id = fields.Integer()
    patent_id = fields.Integer()


class AbstractTextPatchSchema(Schema):
    abstract_text = fields.String()



from datetime import datetime

from sqlalchemy import (
    Column,
    Integer,
    DateTime,
    ForeignKey,
    Boolean,
)

from ...database import Base


class DealConditionModel(Base):
    __tablename__ = "deal_conditions"

    id = Column(Integer, primary_key=True)
    deal_id = Column(
        Integer,
        ForeignKey('deals.id'),
        nullable=False,
    )
    condition_id = Column(
        Integer,
        ForeignKey('conditions.id'),
        nullable=False,
    )
    is_deleted = Column(Boolean, nullable=True)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



from marshmallow import (
    Schema,
    fields,
    validate,
    pre_load,
)

from ...utils.utils import pre_load_date_fields


class PatentResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    patent_number = fields.String(required=True)
    jurisdiction = fields.String(required=True)
    patent_application_id = fields.Integer(allow_none=True)
    appl_id = fields.String(allow_none=True)
    app_grp_art_number = fields.String(allow_none=True)
    app_type = fields.String(allow_none=True)
    country_code = fields.String()
    document_number = fields.String()
    kind_code = fields.String()
    primary_identifier = fields.String()
    abstract_text = fields.String()
    applicant = fields.String()
    inventors = fields.String()
    title = fields.String(allow_none=True)
    url = fields.String(allow_none=True)
    app_class = fields.String(allow_none=True)
    app_sub_class = fields.String(allow_none=True)
    grant_date = fields.DateTime(allow_none=True)
    submission_date = fields.DateTime(allow_none=True)
    app_filing_date = fields.DateTime(allow_none=True)
    app_status_date = fields.DateTime(allow_none=True)
    app_early_pub_date = fields.DateTime(allow_none=True)
    filing_date_us = fields.DateTime(allow_none=True)
    expiration_date = fields.DateTime(allow_none=True)
    pto_adjustments = fields.String(allow_none=True)
    appl_delay = fields.String(allow_none=True)
    total_pto_days = fields.String(allow_none=True)
    assignee = fields.String()
    patent_pdf_url = fields.String(allow_none=True)
    espace_url = fields.String(allow_none=True)

    updated_at = fields.DateTime()

    @pre_load
    def convert_string_to_datetime(self, in_data, **kwargs):
        date_fields = [
            'grant_date',
            'submission_date',
            'app_filing_date',
            'app_status_date',
            'app_early_pub_date',
            'filing_date_us',
            'expiration_date',
        ]
        in_data = pre_load_date_fields(
            in_data,
            date_fields,
            date_format='%Y/%m/%d',
        )
        return in_data


class PatentQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer()
    patent_number = fields.String()
    app_grp_art_number = fields.String(validate=not_blank)
    jurisdiction = fields.String(validate=not_blank)
    primary_identifier = fields.String(validate=not_blank)
    applicant = fields.String(validate=not_blank)
    inventors = fields.String(validate=not_blank)


class PatentPatchSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    app_grp_art_number = fields.String(allow_none=True)
    app_type = fields.String(allow_none=True)
    appl_id = fields.String(allow_none=True)
    country_code = fields.String()
    document_number = fields.String()
    kind_code = fields.String()
    primary_identifier = fields.String()
    abstract_text = fields.String()
    applicant = fields.String()
    inventors = fields.String()
    title = fields.String(allow_none=True)
    url = fields.String(allow_none=True)
    grant_date = fields.DateTime(allow_none=True)
    submission_date = fields.DateTime(allow_none=True)
    app_filing_date = fields.DateTime(allow_none=True)
    app_status_date = fields.DateTime(allow_none=True)
    app_early_pub_date = fields.DateTime(allow_none=True)
    filing_date_us = fields.DateTime(allow_none=True)
    expiration_date = fields.DateTime(allow_none=True)
    pto_adjustments = fields.String(allow_none=True)
    appl_delay = fields.String(allow_none=True)
    total_pto_days = fields.String(allow_none=True)
    assignee = fields.String()
    patent_pdf_url = fields.String(allow_none=True)
    espace_url = fields.String(allow_none=True)



from sqlalchemy import (
    Column,
    Integer,
    String,
    Text,
    ForeignKey,
)

from ...aact_database import Base


class BriefSummaryModel(Base):
    __tablename__ = 'facilities'
    __table_args__ = (
        {'schema': 'ctgov'},
    )

    id = Column(Integer, primary_key=True)
    nct_id = Column(
        String,
        ForeignKey('ctgov.studies.nct_id'),
        nullable=False,
    )
    status = Column(String)
    name = Column(String)
    city = Column(String)
    state = Column(String)
    zip = Column(String)
    country = Column(String)

    def __repr__(self):
        return "<Facility(id='{}', nct_id='{}', status='{}', name='{}', city='{}', state='{}', zip='{}', country='{}'>"\
            .format(self.id, self.nct_id, self.status, self.name, self.city, self.state, self.zip, self.country)



from sqlalchemy import (
    Integer,
    Column,
    String,
    DateTime,
    Text,
    ForeignKey,
)
from datetime import datetime
from ...database import Base


class NCTDesignOutcomesModel(Base):
    __tablename__ = "nct_design_outcomes"

    id = Column(Integer, primary_key=True)
    nct_study_id = Column(
        Integer,
        ForeignKey('nct_study.id'),
        nullable=False,
    )
    outcome_type = Column(String(128))
    measure = Column(Text)
    time_frame = Column(String(191))
    description = Column(Text)
    last_update_submitted_qc_date = Column(DateTime)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



from datetime import datetime
from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    UniqueConstraint,
    ForeignKey,
)

from ...database import Base


class PatentIdentifierSynonymModel(Base):
    __tablename__ = 'patent_identifier_synonyms'

    id = Column(Integer, primary_key=True)
    patent_id = Column(
        Integer,
        ForeignKey('patents.id'),
        nullable=False,
    )
    synonym = Column(String(128), nullable=False)

    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )
    __table_args__ = (UniqueConstraint('patent_id', 'synonym'),)



from marshmallow import (
    Schema,
    fields,
    validate,
)


class PatentTargetResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    patent_id = fields.Integer(required=True)
    target_id = fields.Integer(required=True)
    score = fields.Float(required=True)
    preferred = fields.Boolean(allow_none=True)
    updated_at = fields.DateTime()



from sqlalchemy.exc import SQLAlchemyError
from marshmallow import ValidationError, EXCLUDE

from ...database import db_session
from ...resources.ProceedingStage.models import ProceedingStageModel
from .schemas import (
    ProceedingStageResourceSchema,
)


schema_resource = ProceedingStageResourceSchema()


class ProceedingStage(object):
    @staticmethod
    def create(params):
        """
        Args:
            params: Dict (ProceedingStageResourceSchema)

        Returns:
            ProceedingStageResourceSchema

        Raises:
            ValidationError
            DBException
        """
        data = schema_resource.load(params, unknown=EXCLUDE)
        response = _helper_create(data)
        return response

    @staticmethod
    def delete(id):
        """
        :param
            id
        :return:
            delete message
        :exception:
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            SQLAlchemyError
        """
        proceeding_stage_query = db_session.query(
            ProceedingStageModel).filter_by(id=id).one()
        try:
            db_session.delete(proceeding_stage_query)
            db_session.commit()
            db_session.close()
            return 'Successfully deleted'
        except SQLAlchemyError:
            db_session.rollback()
            db_session.close()
            raise

    @staticmethod
    def bulk_delete(ptab2_proceeding_id):
        try:
            db_session.query(
                ProceedingStageModel).filter(
                ProceedingStageModel.ptab2_proceeding_id == ptab2_proceeding_id
            ).delete(synchronize_session=False)
            db_session.commit()
            db_session.close()
            return 'Successfully deleted'
        except SQLAlchemyError:
            db_session.rollback()
            db_session.close()
            raise


def _helper_create(data):
    new_proceeding_stage = ProceedingStageModel(**data)
    try:
        db_session.add(new_proceeding_stage)
        db_session.commit()
        proceeding_stage_query = db_session.query(
            ProceedingStageModel).get(new_proceeding_stage.id)
        response = schema_resource.dump(proceeding_stage_query)
        db_session.close()
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise



from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    ForeignKey,
    Float,
)
from datetime import datetime

from ...database import Base


class TagTreeModel(Base):
    __tablename__ = 'tag_tree'

    id = Column(Integer, primary_key=True)
    tag_id = Column(
        Integer,
        ForeignKey('news_tags.id'),
        nullable=False,
    )
    label_name = Column(String(50), nullable=False)
    score = Column(Float, nullable=False)
    method = Column(String(50), nullable=False)
    level = Column(Integer, nullable=False)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



from sqlalchemy import (
    Column,
    Integer,
    String,
    Text,
    DateTime,
)
from datetime import datetime

from ...database import Base


class PTAB2DecisionModel(Base):
    __tablename__ = 'ptab2_decisions'

    id = Column(Integer, primary_key=True)
    board_rulings = Column(Text)
    decision_date = Column(DateTime)
    decision_type_category = Column(String(256))
    document_identifier = Column(String(256))
    document_name = Column(String(256))
    identifier = Column(String(100), nullable=False, unique=True)
    issue_type = Column(Text)
    petitioner_application_number_text = Column(String(256))
    petitioner_counsel_name = Column(String(256))
    petitioner_grant_date = Column(DateTime)
    petitioner_group_art_unit_number = Column(String(256))
    petitioner_inventor_name = Column(String(256))
    petitioner_party_name = Column(String(256))
    petitioner_patent_number = Column(String(50))
    petitioner_patent_owner_name = Column(String(256))
    petitioner_technology_center_number = Column(String(256))
    proceeding_number = Column(String(20))
    proceeding_type_category = Column(String(20))
    respondent_application_number_text = Column(String(50))
    respondent_counsel_name = Column(String(100))
    respondent_grant_date = Column(DateTime)
    respondent_group_art_unit_number = Column(String(50))
    respondent_inventor_name = Column(String(100))
    respondent_party_name = Column(String(256))
    respondent_patent_number = Column(String(20))
    respondent_patent_owner_name = Column(String(256))
    respondent_technology_center_number = Column(String(20))
    subdecision_type_category = Column(String(100))
    subproceeding_type_category = Column(String(100))
    claims_invalidation_status = Column(Text)
    updated_at = Column(
        DateTime,
        nullable=False,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )



from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm.exc import NoResultFound
from marshmallow import ValidationError, EXCLUDE

from cs_models.utils.utils import update_model
from ...database import db_session
from ...resources.Newswire.models import (
    NewswireModel,
)
from .schemas import (
    NewswireResourceSchema,
    NewswireQueryParamsSchema,
    NewswirePatchSchema,
)


schema_resource = NewswireResourceSchema()
schema_params = NewswireQueryParamsSchema()
schema_patch = NewswirePatchSchema()


class Newswire(object):
    @staticmethod
    def create(params):
        """
        Args:
            params: dict(NewswireResourceSchema)

        Returns: NewswireResourceSchema

        Raises:
            ValidationError
            SQLAlchemyError
        """
        data = schema_resource.load(params, unknown=EXCLUDE)
        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        Args:
            params: dict(NewswireQueryParamsSchema)

        Returns: List<NewswireResourceSchema>

        Raises:
            ValidationError
        """
        data = schema_params.load(params, unknown=EXCLUDE)

        document_ref_query = _build_query(params=data)
        response = schema_resource.dump(
            document_ref_query, many=True)
        return response

    @staticmethod
    def one(params):
        """
        Args:
            params: dict(NewswireQueryParamsSchema)

        Returns: NewswireResourceSchema

        Raises:
            ValidationError
        """
        data = schema_params.load(params, unknown=EXCLUDE)

        newswire_query = _build_query(params=data).one()
        response = schema_resource.dump(newswire_query)
        return response

    @staticmethod
    def update(id, params):
        """

        Args:
            id: int
            params: NewswirePatchSchema

        Returns:
            NewswireResourceSchema

        Raises:
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            ValidationError
        """
        newswire_query = db_session.query(
            NewswireModel).filter_by(id=id).one()
        data = schema_patch.load(params, unknown=EXCLUDE)

        response = _helper_update(data, newswire_query)
        return response

    @staticmethod
    def upsert(params):
        """
        Args:
            params: NewswireResourceSchema

        Returns:
            NewswireResourceSchema

        Raises:
            ValidationError
        """
        data = schema_resource.load(params, unknown=EXCLUDE)
        try:
            query_params = {
                'headline': params['headline'],
                'subsidiary_id': params['subsidiary_id'],
            }
            newswire_query = _build_query(query_params).one()
            response = _helper_update(data, newswire_query)
        except NoResultFound:
            response = _helper_create(data)
        return response

    @staticmethod
    def delete(id):
        """
        Args:
            id: int

        Returns: string

        Raises:
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            SQLAlchemyError
        """
        newswire_query = db_session.query(
            NewswireModel).filter_by(id=id).one()
        try:
            db_session.delete(newswire_query)
            db_session.commit()
            db_session.close()
            return 'Successfully deleted'
        except SQLAlchemyError:
            db_session.rollback()
            db_session.close()
            raise


def _helper_create(data):
    new_newswire = NewswireModel(**data)
    try:
        db_session.add(new_newswire)
        db_session.commit()
        newswire_query = db_session.query(
            NewswireModel).get(
            new_newswire.id)
        response = schema_resource.dump(newswire_query)
        db_session.close()
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise


def _helper_update(data, newswire_query):
    data['id'] = newswire_query.id
    try:
        update_model(data, newswire_query)
        db_session.commit()
        response = schema_resource.dump(newswire_query)
        return response
    except SQLAlchemyError:
        db_session.rollback()
        raise


def _build_query(params):
    q = db_session.query(
        NewswireModel)
    if params.get('id'):
        q = q.filter_by(id=params.get('id'))
    if params.get('date'):
        q = q.filter_by(date=params.get('date'))
    if params.get('headline'):
        q = q.filter_by(headline=params.get('headline'))
    if params.get('source'):
        q = q.filter_by(source=params.get('source'))
    if params.get('subsidiary_id'):
        q = q.filter_by(subsidiary_id=params.get('subsidiary_id'))
    return q



from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    ForeignKey,
)
from datetime import datetime

from ...database import Base


class TargetSynModel(Base):
    __tablename__ = 'target_syns'

    id = Column(Integer, primary_key=True)
    synonym = Column(String(191), nullable=False, index=True)
    target_id = Column(
        Integer,
        ForeignKey('targets.id'),
        nullable=True,
    )
    updated_at = Column(
        DateTime,
        nullable=False,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )



from datetime import datetime

from sqlalchemy import (
    Column,
    Integer,
    DateTime,
    String,
    Float,
    Boolean,
)

from ...database import Base


class InterventionDataRawModel(Base):
    __tablename__ = "intervention_data_raw"

    id = Column(Integer, primary_key=True)
    source_type = Column(
        String(50),
        nullable=False
    )
    source_id = Column(Integer, nullable=False)
    insight_type = Column(
        String(50), nullable=False,
    )
    insight_name = Column(
        String(191),
    )
    score = Column(Float, nullable=True)
    preferred = Column(Boolean, nullable=True)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm.exc import NoResultFound
from marshmallow import ValidationError, EXCLUDE

from ...database import db_session
from ...resources.OrangeBookExclusivity.models import (
    OrangeBookExclusivityModel,
)
from .schemas import (
    OrangeBookExclusivityResourceSchema,
    OrangeBookExclusivityQueryParamsSchema,
)
from ...utils.utils import update_model


schema_resource = OrangeBookExclusivityResourceSchema()
schema_params = OrangeBookExclusivityQueryParamsSchema()


class OrangeBookExclusivity(object):
    @staticmethod
    def create(params):
        """
        Args:
            params: dict(OrangeBookExclusivityResourceSchema)

        Returns: OrangeBookExclusivityResourceSchema

        Raises:
            ValidationError
            SQLAlchemyError
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)
        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        Args:
            params: dict(OrangeBookExclusivityQueryParamsSchema)

        Returns: List<OrangeBookExclusivityResourceSchema>

        Raises:
            ValidationError
        """
        data = schema_params.load(params, unknown=EXCLUDE)

        ob_exclusivity_query = _build_query(params=data)
        response = schema_resource.dump(
            ob_exclusivity_query, many=True)
        return response

    @staticmethod
    def one(params):
        """
        Args:
            params: dict(OrangeBookExclusivityQueryParamsSchema)

        Returns: OrangeBookExclusivityResourceSchema

        Raises:
            ValidationError
        """
        data = schema_params.load(params, unknown=EXCLUDE)

        ob_exclusivity_query = _build_query(params=data).one()
        response = schema_resource.dump(ob_exclusivity_query)
        return response

    @staticmethod
    def upsert(params):
        """
        Args:
            params: OrangeBookExclusivityResourceSchema

        Returns:
            OrangeBookExclusivityResourceSchema

        Raises:
            ValidationError
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)

        try:
            query_params = {
                'appl_type': params['appl_type'],
                'appl_no': params['appl_no'],
                'product_no': params['product_no'],
                'exclusivity_code': params['exclusivity_code'],
            }
            ob_exclusivity_query = _build_query(query_params).one()
            response = _helper_update(data, ob_exclusivity_query)
        except NoResultFound:
            response = _helper_create(data)
        return response


def _helper_create(data):
    new_ob_exclusivity = OrangeBookExclusivityModel(**data)
    try:
        db_session.add(new_ob_exclusivity)
        db_session.commit()
        ob_exclusivity_query = db_session.query(
            OrangeBookExclusivityModel).get(new_ob_exclusivity.id)
        response = schema_resource.dump(ob_exclusivity_query)
        db_session.close()
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise


def _helper_update(data, ob_exclusivity_query):
    data['id'] = ob_exclusivity_query.id
    data['appl_no'] = ob_exclusivity_query.appl_no
    data['product_no'] = ob_exclusivity_query.product_no
    data['exclusivity_code'] = ob_exclusivity_query.exclusivity_code

    data['exclusivity_date'] = (
        data['exclusivity_date']
        if data['exclusivity_date'] > ob_exclusivity_query.exclusivity_date
        else ob_exclusivity_query.exclusivity_date
    )

    try:
        update_model(data, ob_exclusivity_query)
        db_session.commit()
        response = schema_resource.dump(ob_exclusivity_query)
        return response
    except SQLAlchemyError:
        db_session.rollback()
        raise


def _build_query(params):
    q = db_session.query(
        OrangeBookExclusivityModel)
    if params.get('id'):
        q = q.filter_by(id=params.get('id'))
    if params.get('appl_type'):
        q = q.filter_by(appl_type=params.get('appl_type'))
    if params.get('appl_no'):
        q = q.filter_by(appl_no=params.get('appl_no'))
    if params.get('product_no'):
        q = q.filter_by(product_no=params.get('product_no'))
    if params.get('exclusivity_code'):
        q = q.filter_by(exclusivity_code=params.get('exclusivity_code'))
    return q



from marshmallow import (
    Schema,
    fields,
    validate,
    pre_load,
    ValidationError,
)


class CompanyScientificLinkOutboxResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    company_sec_id = fields.Integer(allow_none=True)
    company_ous_id = fields.Integer(allow_none=True)
    webpage_url = fields.String(required=True)
    start_url = fields.String(required=True)
    has_link = fields.Boolean(allow_none=True)
    updated_at = fields.DateTime(dump_only=True)

    @pre_load
    def check_company_ids(self, in_data, **kwargs):
        if self._get_number_of_company_fields(in_data) > 1:
            raise ValidationError('Provide either company_sec_id or '
                                  'company_ous_id, not both')
        return in_data

    def _get_number_of_company_fields(self, in_data, **kwargs):
        result = 0
        if 'company_sec_id' in in_data:
            if in_data['company_sec_id'] is not None:
                result += 1
        if 'company_ous_id' in in_data:
            if in_data['company_ous_id'] is not None:
                result += 1
        return result



from sqlalchemy import (
    Column,
    Integer,
    String,
    Text,
    DateTime,
    UniqueConstraint,
)
from datetime import datetime

from ...database import Base


class CrossRefModel(Base):
    __tablename__ = 'cross_refs'

    id = Column(Integer, primary_key=True)
    doi = Column(String(191), nullable=False)
    authors = Column(Text)
    doi_url = Column(String(500))
    issn = Column(Text)
    issue = Column(String(10))
    journal = Column(String(500))
    pages = Column(String(128))
    pmid = Column(String(50))
    ref_type = Column(String(128))
    text = Column(Text)
    title = Column(Text)
    volume = Column(String(50))
    year = Column(Integer)
    updated_at = Column(
        DateTime,
        nullable=False,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )

    __table_args__ = (UniqueConstraint('doi'),)



import copy
from marshmallow import ValidationError
from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm.exc import (
    NoResultFound,
)
from parameterized import parameterized

from test.backendtestcase import TestCase
from test.utils import second_equals_first
from src.cs_models.resources.PTAB2Proceeding import PTAB2Proceeding
from src.cs_models.resources.Subsidiary import Subsidiary
from src.cs_models.resources.CompanySEC import CompanySEC
from src.cs_models.resources.CompanyOUS import CompanyOUS


class PTAB2ProceedingResourceTestCase(TestCase):
    def setUp(self):
        super(PTAB2ProceedingResourceTestCase, self).setUp()
        self.inst_subsidiary = Subsidiary()
        self.inst_sec_company = CompanySEC()
        self.inst_ous_company = CompanyOUS()
        self.inst = PTAB2Proceeding()

        self.company_sec1 = self.inst_sec_company.create({
            'cik_str': '200406',
            'ticker': 'JNJ',
            "title": "JOHNSON & JOHNSON",
        })

        self.company_sec2 = self.inst_sec_company.create({
            "cik_str": "816284",
            "ticker": "CELG",
            "title": "CELGENE CORP /DE/"
        })

        self.company_ous1 = self.inst_ous_company.create({
            'name': 'Aurobindo Pharma',
            'ticker': 'AUROPHARMA',
            'exchange': 'NSE',
        })

        self.subsidiary1 = self.inst_subsidiary.create({
            'name': 'Janssen Scientific Affairs, LLC',
            'company_sec_id': self.company_sec1['id'],
            'company_ous_id': None,
        })

        self.subsidiary2 = self.inst_subsidiary.create({
            'name': 'Celgene International II SARL',
            'company_sec_id': self.company_sec2['id'],
            'company_ous_id': None,
        })

        self.ptab2_proceeding1 = self.inst.create({
            'accorded_filing_date': '05-06-2013',
            'decision_date': '10-07-2014',
            'institution_decision_date': '10-08-2013',
            'petitioner_application_number_text': '-',
            'petitioner_counsel_name': 'Steven Baughman',
            'petitioner_grant_date': '-',
            'petitioner_group_art_unit_number': '-',
            'petitioner_inventor_name': '-',
            'petitioner_party_name': 'Apple Inc.',
            'petitioner_patent_number': '-',
            'petitioner_patent_owner_name': '-',
            'petitioner_technology_center_number': '-',
            'proceeding_filing_date': '05-06-2013',
            'proceeding_last_modified_date': '-',
            'proceeding_number': 'IPR-100',
            'proceeding_status_category': 'FWD Entered',
            'proceeding_type_category': 'AIA Trial',
            'respondent_application_number_text': '08471964',
            'respondent_counsel_name': 'David Marsh',
            'respondent_grant_date': '10-12-1999',
            'respondent_group_art_unit_number': '-',
            'respondent_inventor_name': 'ARTHUR HAIR',
            'respondent_party_name': 'SightSound Technologies LLC',
            'respondent_patent_number': '5966440',
            'respondent_patent_owner_name': 'SightSound Technologies LLC',
            'respondent_technology_center_number': '2100',
            'subproceeding_type_category': 'CBM',
            'respondent_subsidiary_id': None,
            'petitioner_subsidiary_id': None,
        })

        self.ptab2_proceeding2 = self.inst.create({
            'accorded_filing_date': '05-06-2013',
            'decision_date': '10-07-2014',
            'institution_decision_date': '10-08-2013',
            'petitioner_application_number_text': '-',
            'petitioner_counsel_name': 'Steven Baughman',
            'petitioner_grant_date': '-',
            'petitioner_group_art_unit_number': '-',
            'petitioner_inventor_name': '-',
            'petitioner_party_name': 'Apple Inc.',
            'petitioner_patent_number': '-',
            'petitioner_patent_owner_name': '-',
            'petitioner_technology_center_number': '-',
            'proceeding_filing_date': '05-06-2013',
            'proceeding_last_modified_date': '-',
            'proceeding_number': 'IPR-200',
            'proceeding_status_category': 'FWD Entered',
            'proceeding_type_category': 'AIA Trial',
            'respondent_application_number_text': '08471964',
            'respondent_counsel_name': 'David Marsh',
            'respondent_grant_date': '10-12-1999',
            'respondent_group_art_unit_number': '-',
            'respondent_inventor_name': 'ARTHUR HAIR',
            'respondent_party_name': 'SightSound Technologies LLC',
            'respondent_patent_number': '5966440',
            'respondent_patent_owner_name': 'SightSound Technologies LLC',
            'respondent_technology_center_number': '2100',
            'subproceeding_type_category': 'CBM',
            'proceeding_status': None,
            'respondent_subsidiary_id': self.subsidiary1['id'],
            'petitioner_subsidiary_id': self.subsidiary2['id'],
        })

        self.valid_data = {
            'accorded_filing_date': '05-06-2013',
            'decision_date': '10-07-2014',
            'institution_decision_date': '10-08-2013',
            'petitioner_application_number_text': '-',
            'petitioner_counsel_name': 'Steven Baughman',
            'petitioner_grant_date': '-',
            'petitioner_group_art_unit_number': '-',
            'petitioner_inventor_name': '-',
            'petitioner_party_name': 'Apple Inc.',
            'petitioner_patent_number': '-',
            'petitioner_patent_owner_name': '-',
            'petitioner_technology_center_number': '-',
            'proceeding_filing_date': '05-06-2013',
            'proceeding_last_modified_date': '-',
            'proceeding_number': 'IPR-300',
            'proceeding_status_category': 'FWD Entered',
            'proceeding_type_category': 'AIA Trial',
            'respondent_application_number_text': '08471964',
            'respondent_counsel_name': 'David Marsh',
            'respondent_grant_date': '10-12-1999',
            'respondent_group_art_unit_number': '-',
            'respondent_inventor_name': 'ARTHUR HAIR',
            'respondent_party_name': 'SightSound Technologies LLC',
            'respondent_patent_number': '5966440',
            'respondent_patent_owner_name': 'SightSound Technologies LLC',
            'respondent_technology_center_number': '2100',
            'subproceeding_type_category': 'CBM',
            'proceeding_status': None,
            'respondent_subsidiary_id': self.subsidiary1['id'],
            'petitioner_subsidiary_id': self.subsidiary2['id'],
        }

    def test_create_violates_unique_constraint(self):
        data = copy.copy(self.valid_data)
        data['proceeding_number'] = self.ptab2_proceeding1['proceeding_number']
        self.assertRaises(
            IntegrityError,
            self.inst.create,
            data,
        )

    def test_create(self):
        resp = self.inst.create(self.valid_data)
        expected_data = {
            **self.valid_data,
            **{
                'accorded_filing_date': '2013-05-06T00:00:00+00:00',
                'decision_date': '2014-10-07T00:00:00+00:00',
                'institution_decision_date': '2013-10-08T00:00:00+00:00',
                'petitioner_grant_date': None,
                'proceeding_filing_date': '2013-05-06T00:00:00+00:00',
                'respondent_grant_date': '1999-10-12T00:00:00+00:00',
            }
        }
        second_equals_first(expected_data, resp)

    def test_read(self):
        resp = self.inst.read({})
        self.assertEqual(len(resp), 2)

    @parameterized.expand([
        ('id', 'ptab2_proceeding1', 'id', 1),
        ('proceeding_number', 'ptab2_proceeding1', 'proceeding_number', 1),
    ])
    def test_read_w_params(
        self,
        field_name,
        attr,
        attr_field,
        expected_length,
    ):
        resp = self.inst.read({})
        self.assertEqual(len(resp), 2)

        resp = self.inst.read({
            field_name: getattr(self, attr)[attr_field],
        })
        self.assertEqual(expected_length, len(resp))

    @parameterized.expand([
        ('id', 999, NoResultFound),
    ])
    def test_one_raises_exception(self, field_name, field_value, exception):
        self.assertRaises(
            exception,
            self.inst.one,
            {
                field_name: field_value,
            },
        )

    @parameterized.expand([
        ('id',),
        ('proceeding_number',),
    ])
    def test_one(self, field_name):
        resp = self.inst.one({
            field_name: self.ptab2_proceeding1[field_name],
        })
        second_equals_first(
            self.ptab2_proceeding1,
            resp,
        )

    def test_delete_not_found(self):
        invalid_id = 99999
        self.assertRaises(
            NoResultFound,
            self.inst.delete,
            invalid_id,
        )

    def test_delete(self):
        response = self.inst.one({'id': self.ptab2_proceeding1['id']})
        self.inst.delete(id=response['id'])
        self.assertRaises(
            NoResultFound,
            self.inst.one,
            {'id': self.ptab2_proceeding1['id']},
        )

    def test_upsert_validation_error(self):
        self.assertRaises(
            ValidationError,
            self.inst.upsert,
            {
                'respondent_patent_number': (
                    self.valid_data['respondent_patent_number']),
            }
        )

    def test_upsert_creates_new_entry(self):
        data = copy.copy(self.valid_data)
        self.assertEqual(2, len(self.inst.read({})))
        self.inst.upsert(data)
        self.assertEqual(3, len(self.inst.read({})))

    def test_upsert_updates_existing_row(self):
        data = {
            **self.valid_data,
            **{'proceeding_number': (
                self.ptab2_proceeding1['proceeding_number'])},
        }
        resp = self.inst.upsert(data)
        expected_data = {
            **data,
            **{
                'accorded_filing_date': '2013-05-06T00:00:00+00:00',
                'decision_date': '2014-10-07T00:00:00+00:00',
                'institution_decision_date': '2013-10-08T00:00:00+00:00',
                'petitioner_grant_date': None,
                'proceeding_filing_date': '2013-05-06T00:00:00+00:00',
                'respondent_grant_date': '1999-10-12T00:00:00+00:00',
            }
        }
        second_equals_first(
            expected_data,
            resp,
        )
        self.assertEqual(2, len(self.inst.read({})))

    @parameterized.expand([
        ('respondent_subsidiary_id',),
        ('petitioner_subsidiary_id',),
    ])
    def test_upsert_with_empty_respondent_subsidiary_id(self, field):
        data = {
            **self.valid_data,
            **{field: None},
        }
        self.assertEqual(2, len(self.inst.read({})))
        resp = self.inst.upsert(data)
        self.assertEqual(3, len(self.inst.read({})))
        self.assertIsNone(resp[field])

    @parameterized.expand([
        ('respondent_subsidiary_id',),
        ('petitioner_subsidiary_id',),
    ])
    def test_upsert_violates_fk_for_invalid_company_id(self, field):
        invalid_subsidiary_id = 912839123
        data = {
            **self.valid_data,
            **{field: invalid_subsidiary_id},
        }
        self.assertRaises(
            IntegrityError,
            self.inst.upsert,
            data,
        )

    def test_upsert_with_valid_subsidiary_id(self):
        data = {
            **self.valid_data,
            **{
                'respondent_subsidiary_id': self.subsidiary1['id'],
                'petitioner_subsidiary_id': self.subsidiary2['id'],
            },
        }
        resp = self.inst.upsert(data)
        self.assertEqual(self.subsidiary1['id'], resp[
            'respondent_subsidiary_id'])

    def test_update_fails_when_missing_proceeding_number(self):
        self.assertRaises(
            ValidationError,
            self.inst.update,
            {
                'respondent_patent_number': (
                    self.valid_data['respondent_patent_number']),
            }
        )

    def test_partial_update(self):
        self.assertEqual(
            None,
            self.ptab2_proceeding1['proceeding_status'],
        )
        new_proceeding_status = 'INSTITUTED'
        data = {
            'proceeding_number': self.ptab2_proceeding1['proceeding_number'],
            'proceeding_status': new_proceeding_status,
        }
        response = self.inst.update(data)

        expected_result = {
            **self.ptab2_proceeding1,
            **data,
        }
        second_equals_first(
            expected_result,
            response,
            fields_to_ignore=['updated_at'],
        )



from marshmallow import (
    Schema,
    fields,
    validate,
    pre_load,
    ValidationError,
)
from ...utils.utils import pre_load_date_fields


class ConditionEpidemiologyResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    source_date = fields.DateTime(required=True)
    source_title = fields.String(required=True)
    source_name = fields.String(required=True)
    source_table = fields.String(required=True)
    source_table_id = fields.Integer(required=True)
    pubmed_id = fields.Integer(allow_none=True)
    news_id = fields.Integer(allow_none=True)
    measure = fields.String(required=True)
    geographic_area = fields.String(allow_none=True)
    snippet = fields.String(required=True)
    statistic = fields.Float(allow_none=True)
    statistic_text = fields.String(allow_none=True)
    year = fields.String(allow_none=True)
    population_type = fields.String(allow_none=True)
    updated_at = fields.DateTime()

    @pre_load
    def check_resource_info(self, in_data, **kwargs):
        if self._get_number_of_source_fields(in_data) != 1:
            raise ValidationError('Provide only one source info')
        return in_data

    def _get_number_of_source_fields(self, in_data, **kwargs):
        result = 0
        if 'news_id' in in_data:
            if in_data['news_id'] is not None:
                result += 1
        if 'pubmed_id' in in_data:
            if in_data['pubmed_id'] is not None:
                result += 1
        return result

    @pre_load
    def convert_string_to_datetime(self, in_data, **kwargs):
        date_fields = ['source_date']

        in_data = pre_load_date_fields(
            in_data,
            date_fields,
            date_format="%Y%m%dT%H%M%S",
        )
        return in_data



from marshmallow import (
    Schema,
    fields,
    validate,
)


class PatentCompoundResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    patent_id = fields.Integer(required=True)
    cid = fields.String(validate=not_blank, required=True)
    compound_name = fields.String(required=True)
    iupac_name = fields.String(required=True)
    molecular_formula = fields.String(validate=not_blank, required=True)
    molecular_weight = fields.String(validate=not_blank, required=True)
    structure_file_id = fields.Integer(required=True)
    updated_at = fields.DateTime()


class PatentCompoundQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer()
    patent_id = fields.Integer()
    cid = fields.String(validate=not_blank)
    compound_name = fields.String()
    iupac_name = fields.String(validate=not_blank)



from marshmallow import (
    Schema,
    fields,
    validate,
    pre_load
)
from ...utils.utils import pre_load_date_fields


class PubmedResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    pubmed_id = fields.Integer(required=True)
    doi = fields.String(allow_none=True)
    publication_type = fields.String(allow_none=True)
    date_created = fields.DateTime(allow_none=True)
    date_completed = fields.DateTime(allow_none=True)
    date_revised = fields.DateTime(allow_none=True)
    journal_issn = fields.String(allow_none=True)
    journal_volume = fields.String(allow_none=True)
    journal_issue = fields.String(allow_none=True)
    journal_title = fields.String(allow_none=True)
    journal_title_iso_abbrev = fields.String(allow_none=True)
    title = fields.String(required=True)
    abstract = fields.String(allow_none=True)
    author_list = fields.String(allow_none=True)
    keyword_list = fields.String(allow_none=True)
    chemical_list = fields.String(allow_none=True)
    mesh_list = fields.String(allow_none=True)
    coi_statement = fields.String(allow_none=True)
    updated_at = fields.DateTime()

    @pre_load
    def convert_string_to_datetime(self, in_data, **kwargs):
        date_fields = ['date_created', 'date_completed', 'date_revised']

        in_data = pre_load_date_fields(
            in_data,
            date_fields,
            date_format='%Y%m%d',
        )
        return in_data



from sqlalchemy import (
    Column,
    Integer,
    String,
    Text,
    DateTime,
    UniqueConstraint,
    ForeignKey,
)
from datetime import datetime

from ...database import Base


class PatentModel(Base):
    __tablename__ = 'patents'

    id = Column(Integer, primary_key=True)
    patent_number = Column(String(128), nullable=False, index=True)
    jurisdiction = Column(String(128), nullable=False)
    patent_application_id = Column(
        Integer,
        ForeignKey('patent_applications.id'),
        nullable=True,
    )
    appl_id = Column(String(128))
    app_grp_art_number = Column(String(128))
    app_type = Column(String(128))
    country_code = Column(String(128))
    document_number = Column(String(128))
    kind_code = Column(String(128))
    primary_identifier = Column(String(256))
    abstract_text = Column(Text)
    applicant = Column(String(256))
    inventors = Column(String(256))
    title = Column(String(256))
    url = Column(String(500))
    app_class = Column(String(20))
    app_sub_class = Column(String(20))
    grant_date = Column(DateTime)
    submission_date = Column(DateTime)
    app_filing_date = Column(DateTime)
    app_status_date = Column(DateTime)
    app_early_pub_date = Column(DateTime)
    filing_date_us = Column(DateTime)
    expiration_date = Column(DateTime)
    pto_adjustments = Column(String(10))
    appl_delay = Column(String(10))
    total_pto_days = Column(String(10))
    assignee = Column(Text)
    patent_pdf_url = Column(String(500))
    espace_url = Column(String(500))

    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )

    __table_args__ = (UniqueConstraint('patent_number', 'jurisdiction'),)



from marshmallow import (
    Schema,
    fields,
    validate,
)


class PatentIFWResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    patent_id = fields.Integer(allow_none=True)
    patent_application_id = fields.Integer(required=True)
    appl_id = fields.String(required=True)
    document_identifier = fields.String(required=True, validate=not_blank)
    document_code = fields.String(allow_none=True)
    document_description = fields.String(allow_none=True)
    direction_category = fields.String(allow_none=True)
    official_date = fields.DateTime(allow_none=True)
    total_pages = fields.Integer(allow_none=True)
    updated_at = fields.DateTime()



from sqlalchemy.orm.exc import (
    NoResultFound,
)
from parameterized import parameterized

from test.backendtestcase import TestCase
from test.utils import second_equals_first
from src.cs_models.resources.NDCSetIDCross import NDCSetIDCross


class NDCSetIDCrossResourceTestCase(TestCase):
    def setUp(self):
        super(NDCSetIDCrossResourceTestCase, self).setUp()
        self.inst = NDCSetIDCross()

        self.ndcsetidcross1 = self.inst.create({
            "product_ndc": "12745-202",
            "package_ndc": "12745-202-02",
            "spl_set_id": "f6d677b8-fa89-4efb-984a-fd98e3e590e8",

        })

        self.ndcsetidcross2 = self.inst.create({
            "product_ndc": "52862-007",
            "package_ndc": "52862-007-12",
            "spl_set_id": "5c8f7727-5f21-4a69-8af1-c24b43571d4d",

        })

        self.valid_data = {
            "product_ndc": "0363-0871",
            "package_ndc": "0363-0871-43",
            "spl_set_id": "eea239fe-554f-4487-9312-3ab0af27d530",
        }

    def test_create(self):
        resp = self.inst.create(self.valid_data)
        expected_data = {
            **self.valid_data,
        }
        second_equals_first(expected_data, resp)

    def test_read(self):
        resp = self.inst.read({})
        self.assertEqual(2, len(resp))

    @parameterized.expand([
        ('product_ndc', 'ndcsetidcross1', 'product_ndc', 1),
        ('spl_set_id', 'ndcsetidcross1', 'spl_set_id', 1),
    ])
    def test_read_w_params(
        self,
        field_name,
        attr,
        attr_field,
        expected_length,
    ):
        resp = self.inst.read({})
        self.assertEqual(2, len(resp))

        resp = self.inst.read({
            field_name: getattr(self, attr)[attr_field],
        })
        self.assertEqual(expected_length, len(resp))

    @parameterized.expand([
        ('product_ndc', '999', NoResultFound),
    ])
    def test_one_raises_exception(self, field_name, field_value, exception):
        self.assertRaises(
            exception,
            self.inst.one,
            {
                field_name: field_value,
            },
        )

    @parameterized.expand([
        ('product_ndc',),
        ('spl_set_id',),
    ])
    def test_one(self, field_name):
        resp = self.inst.one({
            field_name: self.ndcsetidcross1[field_name],
        })
        second_equals_first(
            self.ndcsetidcross1,
            resp,
        )



from contextlib import contextmanager
from ..graph_database import driver


@contextmanager
def session_scope():
    """Provide a transactional scope around a
    series of operations."""

    session = driver.session()
    try:
        yield session
    except Exception:
        raise
    finally:
        session.close()



from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    Boolean,
    ForeignKey,
)
from datetime import datetime

from ...database import Base


class CompanyOUSModel(Base):
    __tablename__ = 'companies_ous'

    id = Column(Integer, primary_key=True)
    cik = Column(Integer, index=True, nullable=True)
    name = Column(String(190), unique=True, nullable=False)
    ticker = Column(String(20), nullable=True, index=True)
    exchange = Column(String(100), nullable=True)
    company_url = Column(String(255), nullable=True)
    pipeline_url = Column(String(255), nullable=True)
    ir_url = Column(String(255), nullable=True)
    is_activated = Column(Boolean, nullable=True)
    last_crawl_date = Column(DateTime, nullable=True)
    industry_type = Column(String(50), nullable=True)
    company_sec_link = Column(
        Integer,
        ForeignKey('companies_sec.id'),
        nullable=True,
    )
    updated_at = Column(
        DateTime,
        nullable=False,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )



import copy
from marshmallow import ValidationError
from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm.exc import (
    NoResultFound,
)
from parameterized import parameterized

from test.backendtestcase import TestCase
from test.utils import second_equals_first
from src.cs_models.resources.File import File
from src.cs_models.resources.PTAB2Document import PTAB2Document


class PTAB2DocumentResourceTestCase(TestCase):
    def setUp(self):
        super(PTAB2DocumentResourceTestCase, self).setUp()
        self.inst = PTAB2Document()
        self.inst_file = File()

        self.file1 = self.inst_file.create({
            'file_format': 'pdf',
            's3_bucket_name': 'test-bucket',
            's3_key_name': 'key1123',
        })

        self.ptab2_document1 = self.inst.create({
            'document_category': 'Exhibits',
            'document_filing_date': '05-08-2019',
            'document_identifier': '1000',
            'document_name': 'Exhibit 1066 - EC-Naprosyn 2004 label.pdf',
            'document_number': '1066',
            'document_size': '-',
            'document_title_text': 'EC Naprosyn 2004 label',
            'document_type_name': 'EXHIBIT',
            'filing_party_category': '-',
            'media_type_category': '-',
            'petitioner_application_number_text': '-',
            'petitioner_counsel_name': 'Alan H. Pollack',
            'petitioner_grant_date': '-',
            'petitioner_group_art_unit_number': '-',
            'petitioner_inventor_name': '-',
            'petitioner_party_name': 'Mylan Pharmaceuticals Inc.',
            'petitioner_patent_number': '-',
            'petitioner_patent_owner_name': '-',
            'petitioner_technology_center_number': '-',
            'proceeding_number': 'IPR2018-00272',
            'proceeding_type_category': 'AIA Trial',
            'respondent_application_number_text': '14980639',
            'respondent_counsel_name': 'Thomas Blinka',
            'respondent_grant_date': '07-19-2016',
            'respondent_group_art_unit_number': '1617',
            'respondent_inventor_name': '-',
            'respondent_party_name': 'Horizon Pharma USA, Inc.',
            'respondent_patent_number': '9393208',
            'respondent_patent_owner_name': 'AULT et al',
            'respondent_technology_center_number': '1600',
            'subproceeding_type_category': 'IPR',
        })

        self.ptab2_document2 = self.inst.create({
            'document_category': 'Exhibits',
            'document_filing_date': '05-08-2019',
            'document_identifier': '2000',
            'document_name': 'Exhibit 1066 - EC-Naprosyn 2004 label.pdf',
            'document_number': '1066',
            'document_size': '-',
            'document_title_text': 'EC Naprosyn 2004 label',
            'document_type_name': 'EXHIBIT',
            'filing_party_category': '-',
            'media_type_category': '-',
            'petitioner_application_number_text': '-',
            'petitioner_counsel_name': 'Alan H. Pollack',
            'petitioner_grant_date': '-',
            'petitioner_group_art_unit_number': '-',
            'petitioner_inventor_name': '-',
            'petitioner_party_name': 'Mylan Pharmaceuticals Inc.',
            'petitioner_patent_number': '-',
            'petitioner_patent_owner_name': '-',
            'petitioner_technology_center_number': '-',
            'proceeding_number': 'IPR2018-00272',
            'proceeding_type_category': 'AIA Trial',
            'respondent_application_number_text': '14980639',
            'respondent_counsel_name': 'Thomas Blinka',
            'respondent_grant_date': '07-19-2016',
            'respondent_group_art_unit_number': '1617',
            'respondent_inventor_name': '-',
            'respondent_party_name': 'Horizon Pharma USA, Inc.',
            'respondent_patent_number': '9393208',
            'respondent_patent_owner_name': 'AULT et al',
            'respondent_technology_center_number': '1600',
            'subproceeding_type_category': 'IPR',
            'has_smart_doc': True,
        })

        self.valid_data = {
            'document_category': 'Exhibits',
            'document_filing_date': '05-08-2019',
            'document_identifier': '3000',
            'document_name': 'Exhibit 1066 - EC-Naprosyn 2004 label.pdf',
            'document_number': '1066',
            'document_size': '-',
            'document_title_text': 'EC Naprosyn 2004 label',
            'document_type_name': 'EXHIBIT',
            'filing_party_category': '-',
            'media_type_category': '-',
            'petitioner_application_number_text': '-',
            'petitioner_counsel_name': 'Alan H. Pollack',
            'petitioner_grant_date': '-',
            'petitioner_group_art_unit_number': '-',
            'petitioner_inventor_name': '-',
            'petitioner_party_name': 'Mylan Pharmaceuticals Inc.',
            'petitioner_patent_number': '-',
            'petitioner_patent_owner_name': '-',
            'petitioner_technology_center_number': '-',
            'proceeding_number': 'IPR2018-00272',
            'proceeding_type_category': 'AIA Trial',
            'respondent_application_number_text': '14980639',
            'respondent_counsel_name': 'Thomas Blinka',
            'respondent_grant_date': '07-19-2016',
            'respondent_group_art_unit_number': '1617',
            'respondent_inventor_name': '-',
            'respondent_party_name': 'Horizon Pharma USA, Inc.',
            'respondent_patent_number': '9393208',
            'respondent_patent_owner_name': 'AULT et al',
            'respondent_technology_center_number': '1600',
            'subproceeding_type_category': 'IPR',
            'file_id': self.file1['id'],
        }

    def test_create_violates_unique_constraint(self):
        data = copy.copy(self.valid_data)
        data['document_identifier'] = (
            self.ptab2_document1['document_identifier'])
        self.assertRaises(
            IntegrityError,
            self.inst.create,
            data,
        )

    def test_create_violates_fk_constraint(self):
        data = copy.copy(self.valid_data)
        file_id_that_dne = 91923891283
        data['file_id'] = file_id_that_dne
        self.assertRaises(
            Exception,
            self.inst.create,
            data,
        )

    def test_create(self):
        resp = self.inst.create(self.valid_data)
        expected_data = {
            **self.valid_data,
            **{
                'document_filing_date': '2019-05-08T00:00:00+00:00',
                'petitioner_grant_date': None,
                'respondent_grant_date': '2016-07-19T00:00:00+00:00',
            }
        }
        second_equals_first(expected_data, resp)

    def test_read(self):
        resp = self.inst.read({})
        self.assertEqual(len(resp), 2)

    @parameterized.expand([
        ('id', 'ptab2_document1', 'id', 1),
        ('document_identifier', 'ptab2_document1', 'document_identifier', 1),
        ('proceeding_number', 'ptab2_document1', 'proceeding_number', 2),
        ('has_smart_doc', 'ptab2_document2', 'has_smart_doc', 1),
    ])
    def test_read_w_params(
        self,
        field_name,
        attr,
        attr_field,
        expected_length,
    ):
        resp = self.inst.read({})
        self.assertEqual(len(resp), 2)

        resp = self.inst.read({
            field_name: getattr(self, attr)[attr_field],
        })
        self.assertEqual(expected_length, len(resp))

    @parameterized.expand([
        ('id', 999, NoResultFound),
    ])
    def test_one_raises_exception(self, field_name, field_value, exception):
        self.assertRaises(
            exception,
            self.inst.one,
            {
                field_name: field_value,
            },
        )

    @parameterized.expand([
        ('id',),
        ('document_identifier',),
    ])
    def test_one(self, field_name):
        resp = self.inst.one({
            field_name: self.ptab2_document1[field_name],
        })
        second_equals_first(
            self.ptab2_document1,
            resp,
        )

    def test_delete_not_found(self):
        invalid_id = 99999
        self.assertRaises(
            NoResultFound,
            self.inst.delete,
            invalid_id,
        )

    def test_delete(self):
        response = self.inst.one({'id': self.ptab2_document1['id']})
        self.inst.delete(id=response['id'])
        self.assertRaises(
            NoResultFound,
            self.inst.one,
            {'id': self.ptab2_document1['id']},
        )

    def test_update_violates_fk_constraint(self):
        invalid_file_id = 123345
        self.assertRaises(
            IntegrityError,
            self.inst.update,
            self.ptab2_document1['id'],
            {'file_id': invalid_file_id},
        )

    def test_update_raises_validation_error(self):
        invalid_file_id = 'asd'
        self.assertRaises(
            ValidationError,
            self.inst.update,
            self.ptab2_document1['id'],
            {'file_id': invalid_file_id},
        )

    def test_partial_update_file_id(self):
        self.assertIsNone(self.ptab2_document1['file_id'])

        self.inst.update(
            self.ptab2_document1['id'],
            {'file_id': self.file1['id']},
        )

        response = self.inst.one({'id': self.ptab2_document1['id']})
        self.assertEqual(self.file1['id'], response['file_id'])

    def test_partial_update_has_smart_doc(self):
        self.assertFalse(self.ptab2_document1['has_smart_doc'])

        self.inst.update(
            self.ptab2_document1['id'],
            {'has_smart_doc': True},
        )

        response = self.inst.one({'id': self.ptab2_document1['id']})
        self.assertTrue(response['has_smart_doc'])

    def test_upsert_validation_error(self):
        self.assertRaises(
            ValidationError,
            self.inst.upsert,
            {
                'petitioner_party_name': (
                    self.valid_data['petitioner_party_name']),
            }
        )

    def test_upsert_creates_new_entry(self):
        data = copy.copy(self.valid_data)
        self.assertEqual(2, len(self.inst.read({})))
        self.inst.upsert(data)
        self.assertEqual(3, len(self.inst.read({})))

    def test_upsert_updates_existing_row(self):
        data = {
            **self.valid_data,
            **{'document_identifier': (
                self.ptab2_document1['document_identifier'])},
        }
        resp = self.inst.upsert(data)
        expected_data = {
            **data,
            **{
                'document_filing_date': '2019-05-08T00:00:00+00:00',
                'petitioner_grant_date': None,
                'respondent_grant_date': '2016-07-19T00:00:00+00:00',
            }
        }
        second_equals_first(
            expected_data,
            resp,
        )
        self.assertEqual(2, len(self.inst.read({})))

    def test_bulk_create(self):
        mock_document1 = {
            **self.valid_data,
            **{'document_identifier': '1'},
        }
        mock_document2 = {
            **self.valid_data,
            **{'document_identifier': '2'},
        }
        mappings = [
            mock_document1,
            mock_document2,
        ]
        response = self.inst.read({})
        self.assertEqual(2, len(response))

        self.inst.bulk_create(mappings)
        response = self.inst.read({})
        self.assertEqual(4, len(response))



from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm.exc import NoResultFound
from marshmallow import ValidationError, EXCLUDE

from cs_models.utils.utils import update_model
from ...database import db_session
from ...resources.NDC.models import NDCModel
from .schemas import (
    NDCPatchSchema,
    NDCQueryParamsSchema,
    NDCResourceSchema,
)


schema_resource = NDCResourceSchema()
schema_params = NDCQueryParamsSchema()
schema_patch = NDCPatchSchema()


class NDC(object):
    @staticmethod
    def create(params):
        """
        Args:
            params: dict(NDCResourceSchema)

        Returns: NDCResourceSchema

        Raises:
            ValidationError
            SQLAlchemyError
        """
        data = schema_resource.load(params, unknown=EXCLUDE)
        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        Args:
            params: dict(NDCQueryParamsSchema)

        Returns: List<NDCResourceSchema>

        Raises:
            ValidationError
        """
        data = schema_params.load(params, unknown=EXCLUDE)

        ndc_query = _build_query(params=data)
        response = schema_resource.dump(ndc_query, many=True)
        return response

    @staticmethod
    def one(params):
        """
        Args:
            params: dict(NDCQueryParamsSchema)

        Returns: NDCResourceSchema

        Raises:
            ValidationError
        """
        data = schema_params.load(params, unknown=EXCLUDE)

        ndc_query = _build_query(params=data).one()
        response = schema_resource.dump(ndc_query)
        return response

    @staticmethod
    def update(id, params):
        """

        Args:
            id: int
            params: NDCPatchSchema

        Returns:
            NDCResourceSchema

        Raises:
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            ValidationError
        """
        ndc_query = db_session.query(NDCModel).filter_by(id=id).one()
        data = schema_patch.load(params, unknown=EXCLUDE)
        response = _helper_update(data, ndc_query)
        return response

    @staticmethod
    def upsert(params):
        """
        Args:
            params: NDCResourceSchema

        Returns:
            NDCResourceSchema

        Raises:
            ValidationError
        """
        data = schema_resource.load(params, unknown=EXCLUDE)

        try:
            query_params = {
                "spl_id": params["spl_id"],
                "product_ndc": params["product_ndc"],
            }
            ndc_query = _build_query(query_params).one()
            response = _helper_update(data, ndc_query)
        except NoResultFound:
            response = _helper_create(data)
        return response

    @staticmethod
    def delete(id):
        """
        Args:
            id: int

        Returns: string

        Raises:
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            SQLAlchemyError
        """
        ndc_query = db_session.query(NDCModel).filter_by(id=id).one()
        try:
            db_session.delete(ndc_query)
            db_session.commit()
            db_session.close()
            return "Successfully deleted"
        except SQLAlchemyError:
            db_session.rollback()
            db_session.close()
            raise


def _helper_create(data):
    new_ndc = NDCModel(**data)
    try:
        db_session.add(new_ndc)
        db_session.commit()
        druglabel_query = db_session.query(NDCModel).get(new_ndc.id)
        response = schema_resource.dump(druglabel_query)
        db_session.close()
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise


def _helper_update(data, ndc_query):
    data["id"] = ndc_query.id
    try:
        update_model(data, ndc_query)
        db_session.commit()
        response = schema_resource.dump(ndc_query)
        return response
    except SQLAlchemyError:
        db_session.rollback()
        raise


def _build_query(params):
    q = db_session.query(NDCModel)
    if params.get("id"):
        q = q.filter_by(id=params.get("id"))
    if params.get("spl_id"):
        q = q.filter_by(spl_id=params.get("spl_id"))
    if params.get("product_ndc"):
        q = q.filter_by(product_ndc=params.get("product_ndc"))
    return q



from datetime import datetime

from sqlalchemy import (
    Column,
    Integer,
    DateTime,
    ForeignKey,
    Text,
    String,
    DECIMAL,
)

from ...database import Base


class DealOutboxModel(Base):
    __tablename__ = "deal_outbox"

    id = Column(Integer, primary_key=True)
    announcement_date = Column(DateTime, nullable=False)
    news_id = Column(
        Integer,
        ForeignKey('newswires.id'),
        nullable=False,
    )
    round = Column(String(50), nullable=True)
    deal_value = Column(DECIMAL(13, 2), nullable=True)
    currency = Column(String(10), nullable=True)
    company_sec_id = Column(
        Integer,
        ForeignKey('companies_sec.id'),
        nullable=True,
    )
    company_ous_id = Column(
        Integer,
        ForeignKey('companies_ous.id'),
        nullable=True,
    )
    investors = Column(Text, nullable=True)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



from marshmallow import (
    Schema,
    fields,
    validate,
)


class FileResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    file_format = fields.String(validate=not_blank, required=True)
    s3_bucket_name = fields.String(validate=not_blank, required=True)
    s3_key_name = fields.String(validate=not_blank, required=True)
    content_length = fields.Integer(allow_none=True)
    sha_256_hash = fields.String(allow_none=True)
    updated_at = fields.DateTime()


class FileQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer()
    file_format = fields.String(validate=not_blank)
    s3_bucket_name = fields.String(validate=not_blank)
    s3_key_name = fields.String(validate=not_blank)


class FilePatchSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    file_format = fields.String(validate=not_blank)
    s3_bucket_name = fields.String(validate=not_blank)
    s3_key_name = fields.String(validate=not_blank)
    content_length = fields.Integer(allow_none=True)



from marshmallow import ValidationError
from parameterized import parameterized
from sqlalchemy.exc import IntegrityError

from src.cs_models.resources.ClaimChallenged import (
    ClaimChallenged,
)
from test.mixins.ClaimDataMixin import ClaimDataMixin
from test.backendtestcase import TestCase
from test.utils import second_equals_first


class ClaimChallengedResourceTestCase(ClaimDataMixin, TestCase):
    def setUp(self):
        super(ClaimChallengedResourceTestCase, self).setUp()

        self.inst = ClaimChallenged()
        self.claim_challenged1 = self.inst.create({
            'ptab2_proceeding_id': self.ptab2_proceeding1['id'],
            'claim_id': self.claim1['id'],
            'prior_art_combination': 0,
            'prior_art_id': self.prior_art1['id'],
            'prior_art_nature': '',
            'nature_of_challenge': 'OBVIOUSNESS',
        })
        self.claim_challenged2 = self.inst.create({
            'ptab2_proceeding_id': self.ptab2_proceeding1['id'],
            'claim_id': self.claim2['id'],
            'prior_art_combination': 1,
            'prior_art_id': self.prior_art2['id'],
            'prior_art_nature': 'primary',
            'nature_of_challenge': 'ANTICIPATION',
        })

    @parameterized.expand([
        'ptab2_proceeding_id',
        'claim_id',
        'prior_art_combination',
        'prior_art_id',
        'nature_of_challenge',
    ])
    def test_create_raises_validation_error(self, missing_field):
        data = {
            'ptab2_proceeding_id': self.ptab2_proceeding1['id'],
            'claim_id': self.claim2['id'],
            'prior_art_combination': 2,
            'prior_art_id': self.prior_art2['id'],
            'prior_art_nature': '',
            'nature_of_challenge': 'OBVIOUSNESS',
        }
        data.pop(missing_field)
        self.assertRaises(
            ValidationError,
            self.inst.create,
            data,
        )

    def test_create_fails_unique_constraint(self):
        self.assertRaises(
            IntegrityError,
            self.inst.create,
            {
                'ptab2_proceeding_id': self.ptab2_proceeding1['id'],
                'claim_id': self.claim1['id'],
                'prior_art_combination': 0,
                'prior_art_id': self.prior_art1['id'],
                'prior_art_nature': '',
                'nature_of_challenge': 'ANTICIPATION',
            },
        )

    def test_create(self):
        mock_prior_art_combination = 0
        data = {
            'ptab2_proceeding_id': self.ptab2_proceeding1['id'],
            'claim_id': self.claim2['id'],
            'prior_art_combination': mock_prior_art_combination,
            'prior_art_id': self.prior_art1['id'],
            'prior_art_nature': 'primary',
            'nature_of_challenge': 'ANTICIPATION',
        }
        resp = self.inst.create(data)
        second_equals_first(
            {
                'ptab2_proceeding_id': self.ptab2_proceeding1['id'],
                'claim_id': self.claim2['id'],
                'prior_art_combination': mock_prior_art_combination,
                'prior_art_id': self.prior_art1['id'],
                'prior_art_nature': data['prior_art_nature'],
                'nature_of_challenge': data['nature_of_challenge'],
            },
            resp,
        )

    def test_read_all(self):
        resp = self.inst.read({})
        self.assertEqual(len(resp), 2)

    @parameterized.expand([
        ('ptab2_proceeding_id', 2),
        ('claim_id', 1),
        ('prior_art_id', 1),
        ('prior_art_combination', 1),
        ('prior_art_nature', 1),
    ])
    def test_read_w_params(self, query_field, expected_length):
        resp = self.inst.read({
            query_field: self.claim_challenged1[query_field],
        })
        self.assertEqual(expected_length, len(resp))
        second_equals_first(
            self.claim_challenged1,
            resp[0],
        )

    def test_delete(self):
        response = self.inst.read({'id': self.claim_challenged1['id']})
        self.assertEqual(len(response), 1)
        response = self.inst.delete(id=self.claim_challenged1['id'])
        self.assertEqual(response, 'Successfully deleted')
        response = self.inst.read({'id': self.claim_challenged1['id']})
        self.assertEqual(len(response), 0)

    def test_bulk_delete(self):
        resp = self.inst.read({})
        self.assertEqual(2, len(resp))
        self.inst.bulk_delete(ptab2_proceeding_id=self.ptab2_proceeding1['id'])
        resp = self.inst.read({})
        self.assertEqual(0, len(resp))



from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm.exc import NoResultFound
from sqlalchemy.sql.operators import in_op
from marshmallow import ValidationError, EXCLUDE

from ...database import db_session
from ...resources.PacerCase.models import PacerCaseModel
from .schemas import (
    PacerCaseResourceSchema,
    PacerCaseQueryParamsSchema,
    PacerCasePatchSchema,
)
from ...utils.utils import update_model


schema_resource = PacerCaseResourceSchema()
schema_params = PacerCaseQueryParamsSchema()
schema_patch = PacerCasePatchSchema()


class PacerCase:
    @staticmethod
    def create(params):
        """
        Args:
            params: Dict (PacerCaseResourceSchema)

        Returns:
            PacerCaseResourceSchema

        Raises:
            ValidationError
            DBException
        """
        data = schema_resource.load(params, unknown=EXCLUDE)
        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        Args:
            params: PacerCaseQueryParamsSchema

        Returns:
            List<PacerCaseResourceSchema>

        Raises:
            ValidationError
        """
        data = schema_params.load(params, unknown=EXCLUDE)

        pacer_case_query = _build_query(params=data)
        response = schema_resource.dump(pacer_case_query, many=True)
        return response

    @staticmethod
    def one(params):
        """
        Args:
            params: PacerCaseQueryParamsSchema

        Returns:
            PacerCaseResourceSchema

        Raises:
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            ValidationError
        """
        data = schema_params.load(params, unknown=EXCLUDE)

        pacer_case_query = _build_query(params=data).one()
        response = schema_resource.dump(pacer_case_query)
        return response

    @staticmethod
    def update(id, params):
        """

        Args:
            id: int
            params: PacerCasePatchSchema

        Returns:
            PacerCaseResourceSchema

        Raises:
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            ValidationError
        """
        pacer_case_query = db_session.query(
            PacerCaseModel).filter_by(id=id).one()
        data = schema_patch.load(params, unknown=EXCLUDE)

        response = _helper_update(data, pacer_case_query)
        return response

    @staticmethod
    def delete(id):
        """
        :param
            id
        :return:
            delete message
        :exception:
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            SQLAlchemyError
        """

        pacer_case_query = db_session.query(
            PacerCaseModel).filter_by(id=id).one()
        try:
            db_session.delete(pacer_case_query)
            db_session.commit()
            db_session.close()
            return 'Successfully deleted'
        except SQLAlchemyError:
            db_session.rollback()
            db_session.close()
            raise

    @staticmethod
    def upsert(params):
        """
        Args:
            params: PacerCaseResourceSchema

        Returns:
            PacerCaseResourceSchema

        Raises:
            ValidationError
        """
        data = schema_resource.load(params, unknown=EXCLUDE)

        try:
            query_params = {
                'court_id': params['court_id'],
                'pacer_case_external_id': params['pacer_case_external_id'],
            }
            pacer_case_query = _build_query(query_params).one()
            response = _helper_update(data, pacer_case_query)
        except NoResultFound:
            response = _helper_create(data)
        return response


def _helper_create(data):
    new_pacer_case = PacerCaseModel(**data)
    try:
        db_session.add(new_pacer_case)
        db_session.commit()
        pacer_case_query = db_session.query(
            PacerCaseModel).get(new_pacer_case.id)
        response = schema_resource.dump(pacer_case_query)
        db_session.close()
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise


def _helper_update(data, pacer_case_query):
    data['id'] = pacer_case_query.id
    data['court_id'] = pacer_case_query.court_id
    data['pacer_case_external_id'] = pacer_case_query.pacer_case_external_id
    try:
        update_model(data, pacer_case_query)
        db_session.commit()
        response = schema_resource.dump(pacer_case_query)
        return response
    except SQLAlchemyError:
        db_session.rollback()
        raise


def _build_query(params):
    q = db_session.query(
        PacerCaseModel)
    if params.get('id'):
        q = q.filter_by(id=params.get('id'))
    if 'ids' in params:
        q = q.filter(in_op(
            PacerCaseModel.id,
            params['ids'],
        ))
    if params.get('case_no'):
        q = q.filter_by(case_no=params.get('case_no'))
    if params.get('court_id'):
        q = q.filter_by(court_id=params.get('court_id'))
    if params.get('pacer_case_external_id'):
        q = q.filter_by(
            pacer_case_external_id=params.get('pacer_case_external_id'))
    return q



from sqlalchemy import (
    Integer,
    Column,
    String,
    DateTime,
    Text,
    ForeignKey,
)
from datetime import datetime
from ...database import Base


class NCTChangeModel(Base):
    __tablename__ = "nct_changes"

    id = Column(Integer, primary_key=True)
    nct_id = Column(String(50), nullable=False)
    nct_study_id = Column(
        Integer,
        ForeignKey('nct_study.id'),
        nullable=True,
        index=True,
    )
    date = Column(DateTime, nullable=False, index=True)
    change_type = Column(String(50), nullable=False, index=True)
    news_id = Column(
        Integer,
        ForeignKey('newswires.id'),
        nullable=True,
    )
    pubmed_id = Column(
        Integer,
        ForeignKey('pubmed.id'),
        nullable=True,
    )
    old_value = Column(String(256), nullable=True)
    new_value = Column(String(256), nullable=True)
    note = Column(Text, nullable=True)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )
